SystemOrganization addCategory: #'SOUL-Parsers'!

RBPatternParser subclass: #SoulRBPatternParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulRBPatternParser methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
scannerClass
	
	^SoulRBPatternScanner! !

!SoulRBPatternParser methodsFor: 'private-classes' stamp: 'sp 24/10/11 15:26'!
variableNodeClass
	
	^SoulRBPatternVariableNode! !

Scanner subclass: #QuotedCodeSmalltalkTokenScanner
	instanceVariableNames: ''
	classVariableNames: 'QuotedCodeTypeTable'
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!QuotedCodeSmalltalkTokenScanner class methodsFor: 'signal constants' stamp: 'sp 24/10/11 15:26'!
errorSignal
	
	^SoulParseError! !

!QuotedCodeSmalltalkTokenScanner class methodsFor: 'class initialization' stamp: 'sp 24/10/11 15:26'!
initialize
	"Scanner initialize"

	"Spacing underbar ($_), which was the assignment character
	up through Release 4.0 of Smalltalk, has been adopted as a
	valid character for variable names and message selectors."

	
	| newTable |
	newTable := ScannerTable new: 255 withAll: #xDefault.	"default"

	newTable atAllSeparatorsPut: #xDelimiter.
	newTable atAllDigitsPut: #xDigit.
	newTable atAllLettersPut: #xLetter.
	newTable
		at: $_ asInteger
		put: #xLetter.
	newTable
		at: Character space asInteger
		put: #xspace.
	newTable
		at: Character tab asInteger
		put: #xspace.
	'!!%&*+,-/<=>\~'
		do:
			[:bin | 
			newTable
				at: bin asInteger
				put: #xBinary].
	#(16rB1 16rD7 16rB7 16rF7) do: [:bin | newTable at: bin put: #xBinary].	"plus-or-minus"
	"times"
	"centered dot"
	"divide"
	"Other multi-character tokens"

	newTable
		at: $? asInteger
		put: #xSoulVariable.
	newTable
		at: $@ asInteger
		put: #xSoulCompound.
	newTable
		at: $" asInteger
		put: #xDoubleQuote.
	newTable
		at: $$ asInteger
		put: #xDollar.	"dollar sign"

	newTable
		at: $' asInteger
		put: #xSingleQuote.	"Single-character tokens"

	newTable
		at: $# asInteger
		put: #literalQuote.
	newTable
		at: $( asInteger
		put: #leftParenthesis.
	newTable
		at: $) asInteger
		put: #rightParenthesis.
	newTable
		at: $. asInteger
		put: #period.
	newTable
		at: $: asInteger
		put: #colon.
	newTable
		at: $; asInteger
		put: #semicolon.
	newTable
		at: $[ asInteger
		put: #leftBracket.
	newTable
		at: $] asInteger
		put: #rightBracket.
	newTable
		at: ${ asInteger
		put: #leftBrace.
	newTable
		at: $} asInteger
		put: #rightBrace.
	newTable
		at: $^ asInteger
		put: #upArrow.	"spacing circumflex, formerly up arrow"
	"Not currently supported.
	newTable at: 16r2191 put: #upArrow.  ""up arrow""
	newTable at: 16r2190 put: #leftArrow.  ""left arrow""
"

	newTable
		at: $| asInteger
		put: #verticalBar.
	QuotedCodeTypeTable := newTable! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'public access' stamp: 'AndyKellens 11/2/2011 09:53'!
breakIntoTokens: textOrString
	"Answer a sequence of DifferSubstrings for aTextOrString, one substring per token.
	 This is for tokenized comparison of code."

	
	| start s terms prevTokenType |
	terms := OrderedCollection new: textOrString size // 4.
	self scan: (ReadStream on: (s := textOrString string)).
	start := 0.
	prevTokenType := tokenType.
	[start > 0
		ifTrue:
			[| end |
			end := prevEnd.
			[end > 0 and: [(s at: end) isSeparator]] whileTrue: [end := end - 1].
			prevTokenType = #xspace
				ifFalse:
					[terms
						addLast:
							(self
								makeTerm: prevTokenType asSymbol
								with: (textOrString copyFrom: start to: end) asSymbol)]].
	tokenType = #doIt]
		whileFalse:
			[start := mark.
			prevTokenType := tokenType.
			self scanToken].
	^SoulFactory current makeList termSequence: (TermSequence terms: terms)! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'scanner directives' stamp: 'sp 24/10/11 15:26'!
getNextChar
	
	hereChar := source next.
	hereChar == nil ifFalse: [buffer nextPut: hereChar]! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'initialize-release' stamp: 'sp 24/10/11 15:26'!
initScanner
	
	buffer := WriteStream on: (String new: 40).
	saveComments := true.
	typeTable := QuotedCodeTypeTable! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'scanner directives' stamp: 'sp 24/10/11 15:26'!
innerScanUpTo: stopChar ignore: startChar
	
	| level |
	level := 0.
	[hereChar = stopChar and: [level = 0]]
		whileFalse:
			[hereChar = startChar ifTrue: [level := level + 1].
			hereChar = stopChar ifTrue: [level := level - 1].
			hereChar == nil
				ifTrue: [self error: 'Unmatched ' , startChar asString]
				ifFalse: [self getNextChar]]! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeTerm: aFunctorSymbol with: anArgument
	
	^SoulFactory current makeCompound
		functor: (Constant name: aFunctorSymbol)
		termSequence:
			(TermSequence terms: (Array with: (Constant name: anArgument)))! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'scanner directives' stamp: 'sp 24/10/11 15:26'!
scanUpTo: stopChar ignore: startChar
	
	| level |
	level := 0.
	[hereChar = stopChar and: [level = 0]]
		whileFalse:
			[hereChar = startChar ifTrue: [level := level + 1].
			hereChar = stopChar ifTrue: [level := level - 1].
			hereChar = $# ifTrue: [self getNextChar].
			hereChar = $'
				ifTrue:
					[self getNextChar.
					self innerScanUpTo: $' ignore: nil].
			hereChar = $$ ifTrue: [self getNextChar].
			hereChar == nil
				ifTrue: [self error: 'Unmatched ' , startChar asString]
				ifFalse: [self getNextChar]].
	self getNextChar! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'multi-character scans' stamp: 'sp 24/10/11 15:26'!
xSoulCompound
	"form a SOUL compound"

	
	buffer reset.
	self getNextChar.
	self scanUpTo: $( ignore: $@.
	self scanUpTo: $) ignore: $(.
	token := buffer contents.
	tokenType := #xSoulCompound! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'multi-character scans' stamp: 'sp 24/10/11 15:26'!
xSoulVariable
	"form a SOUL variable"

	
	| type char |
	buffer reset.
	buffer nextPut: hereChar.
	[char := source next.
	char notNil
		and:
			[type := typeTable at: char asInteger.
			type == #xLetter or: [type == #xDigit]]]
		whileTrue: [buffer nextPut: char].
	hereChar := char.
	token := buffer contents.
	tokenType := #xSoulVariable! !

!QuotedCodeSmalltalkTokenScanner methodsFor: 'multi-character scans' stamp: 'sp 24/10/11 15:26'!
xspace
	
	tokenType := #xspace.
	token := Character space.
	hereChar := source next! !

SmaCCScanner subclass: #QuotedCodeScanner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!QuotedCodeScanner class methodsFor: 'signal constants' stamp: 'sp 24/10/11 15:26'!
errorSignal
	
	^SoulParseError! !

!QuotedCodeScanner class methodsFor: 'generated-comments' stamp: 'AndyKellens 10/28/2011 15:22'!
scannerDefinitionComment

	"<char> : [^ \?\@\\\'];
<space> : \s;
<var> : \? ([A-Za-z0-9])+;
<type> : \\ ([A-Za-z0-9])+;
<quote> : \';
<compound> : \@;
	
	"! !

!QuotedCodeScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
compound
	
	| token |
	self scanUpTo: $( ignore: $@.
	self scanUpTo: $) ignore: $(.
	token := SmaCCToken
		value: outputStream contents
		start: start
		id: (Array with: self compoundId).
	matchActions := nil.
	returnMatchBlock value: token! !

!QuotedCodeScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
compoundId
	^6! !

!QuotedCodeScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
emptySymbolTokenId
	^18! !

!QuotedCodeScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
errorTokenId
	^19! !

!QuotedCodeScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
innerScanUpTo: stopChar ignore: startChar
	
	| level |
	level := 0.
	[currentCharacter = stopChar and: [level = 0]]
		whileFalse:
			[currentCharacter = startChar ifTrue: [level := level + 1].
			currentCharacter = stopChar ifTrue: [level := level - 1].
			self atEnd
				ifTrue: [self error: 'Unmatched ' , startChar asString]
				ifFalse: [self step]]! !

!QuotedCodeScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scanForToken
	self step.
	(currentCharacter <= Character backspace
		or: [ 
			(currentCharacter between: (Character value: 14) and: Character arrowDown)
				or: [ 
					(currentCharacter between: $!! and: $&)
						or: [ (currentCharacter between: $( and: $>) or: [ (currentCharacter between: $A and: $[) or: [ currentCharacter >= $] ] ] ] ] ])
		ifTrue: [ ^ self recordAndReportMatch: #(1) ].
	(currentCharacter between: Character tab and: Character cr)
		ifTrue: [ ^ self recordAndReportMatch: #(1 2) ].
	currentCharacter = Character space
		ifTrue: [ ^ self recordAndReportMatch: #(2) ].
	currentCharacter = $'
		ifTrue: [ ^ self recordAndReportMatch: #(5) ].
	currentCharacter = $?
		ifTrue: [ 
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(3).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $@
		ifTrue: [ ^ self recordAndReportMatch: #compound ].
	currentCharacter = $\
		ifTrue: [ 
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(4).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	^ self reportLastMatch! !

!QuotedCodeScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
scanUpTo: stopChar ignore: startChar
	
	| level |
	level := 0.
	self step.
	[currentCharacter = stopChar and: [level = 0]]
		whileFalse:
			[currentCharacter = startChar ifTrue: [level := level + 1].
			currentCharacter = stopChar ifTrue: [level := level - 1].
			currentCharacter = $# ifTrue: [self step].
			currentCharacter = $'
				ifTrue:
					[self step.
					self innerScanUpTo: $' ignore: nil].
			currentCharacter = $$ ifTrue: [self step].
			self atEnd
				ifTrue: [self error: 'Unmatched ' , startChar asString]
				ifFalse: [self step]]	"self getNextChar."
	"self token: outputStream contents"! !

SmaCCScanner subclass: #SmalltalkTermScanner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SmalltalkTermScanner class methodsFor: 'generated-initialization' stamp: 'AndyKellens 10/28/2011 15:22'!
initializeKeywordMap
	keywordMap := Dictionary new.
	#(#(20 'false' 12) #(20 'nil' 14) #(20 'true' 11) #(26 '||' 15))
		do: [ :each | (keywordMap at: each first ifAbsentPut: [ Dictionary new ]) at: (each at: 2) put: each last ].
	^ keywordMap! !

!SmalltalkTermScanner class methodsFor: 'generated-comments' stamp: 'AndyKellens 10/28/2011 15:22'!
scannerDefinitionComment

	"<name>	:	[a-zA-Z_] \w*	;
<number>:	[0-9]	;
<negativenumber>	:	\- <number>	;
<string>	:	\'  [^\']* \' (\' [^\']* \')*	;
<keyword>	:	<name> \:	;
<multikeyword>	:	<name> \: (<name> \: )+	;
<binarysymbol>	:	[\~\-\!!\@\%\&\*\+\=\\\|\?\/\>\<\,] [\~\!!\@\%\&\*\+\=\\\|\?\/\>\<\,]?	;
<whitespace>	:	\s+	;
<comment>	:	\"" [^\""]* \""	;
<dottedname>	:	<name> (\. <name>)+	;
<character>	:	\$ .	;
<variableassignment> : <name> \: \=	;
<soulvar> : \? ([A-Za-z0-9])+;
<delayedsoulvar> : \& ([A-Za-z0-9])+;
<anychar> : . ; # For VW literal arrays that handle #(;) -> #(#';')
	
	"! !

!SmalltalkTermScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
binarysymbolId
	^26! !

!SmalltalkTermScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
emptySymbolTokenId
	^66! !

!SmalltalkTermScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
errorTokenId
	^67! !

!SmalltalkTermScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
nameId
	^20! !

!SmalltalkTermScanner methodsFor: 'scanning' stamp: 'AndyKellens 10/28/2011 17:03'!
negativenumber
	
	stream skip: -2.
	self scanNumber: self negativenumberId ! !

!SmalltalkTermScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
negativenumberId
	^22! !

!SmalltalkTermScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
number
	
	stream skip: -1.
	self scanNumber: self numberId! !

!SmalltalkTermScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
numberId
	^21! !

!SmalltalkTermScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scan1
	self recordMatch: #(23).
	self step.
	currentCharacter = $'
		ifTrue: [ ^ self scan2 ].
	^ self reportLastMatch! !

!SmalltalkTermScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scan2
	[ 
	self step.
	currentCharacter ~= $' ] whileTrue.
	currentCharacter = $'
		ifTrue: [ ^ self scan1 ].
	^ self reportLastMatch! !

!SmalltalkTermScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scan3
	self recordMatch: #(24).
	self step.
	((currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ])
		ifTrue: [ ^ self scan4 ].
	currentCharacter = $=
		ifTrue: [ ^ self recordAndReportMatch: #variableassignment ].
	^ self reportLastMatch! !

!SmalltalkTermScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scan4
	[ 
	self step.
	(currentCharacter between: $0 and: $9)
		or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ] ] ]
		whileTrue.
	currentCharacter = $:
		ifTrue: [ 
			self recordMatch: #(25).
			self step.
			((currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ ^ self scan4 ].
			^ self reportLastMatch ].
	^ self reportLastMatch! !

!SmalltalkTermScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scan5
	self step.
	((currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ])
		ifTrue: [ 
			[ 
			self recordMatch: #(29).
			self step.
			(currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ] ] ]
				whileTrue.
			currentCharacter = $.
				ifTrue: [ ^ self scan5 ].
			^ self reportLastMatch ].
	^ self reportLastMatch! !

!SmalltalkTermScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scanForToken
	self step.
	(currentCharacter <= Character backspace
		or: [ 
			(currentCharacter between: (Character value: 14) and: Character arrowDown)
				or: [ currentCharacter = $` or: [ currentCharacter >= Character delete ] ] ])
		ifTrue: [ ^ self recordAndReportMatch: #(34) ].
	((currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ])
		ifTrue: [ 
			self recordMatch: #(20 34).
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(20).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter = $_ or: [ currentCharacter between: $a and: $z ] ] ] ]
						whileTrue.
					currentCharacter = $.
						ifTrue: [ ^ self scan5 ].
					currentCharacter = $:
						ifTrue: [ ^ self scan3 ].
					^ self reportLastMatch ].
			currentCharacter = $.
				ifTrue: [ ^ self scan5 ].
			currentCharacter = $:
				ifTrue: [ ^ self scan3 ].
			^ self reportLastMatch ].
	(currentCharacter = $!!
		or: [ 
			currentCharacter = $%
				or: [ 
					(currentCharacter between: $* and: $,)
						or: [ 
							currentCharacter = $/
								or: [ currentCharacter = $= or: [ currentCharacter = $@ or: [ currentCharacter = $\ or: [ currentCharacter = $~ ] ] ] ] ] ] ])
		ifTrue: [ 
			self recordMatch: #(26 34).
			self step.
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			^ self reportLastMatch ].
	(currentCharacter between: $0 and: $9)
		ifTrue: [ ^ self recordAndReportMatch: #number ].
	((currentCharacter between: Character tab and: Character cr) or: [ currentCharacter = Character space ])
		ifTrue: [ 
			self recordMatch: #whitespace.
			self step.
			((currentCharacter between: Character tab and: Character cr) or: [ currentCharacter = Character space ])
				ifTrue: [ 
					[ 
					self recordMatch: #whitespace.
					self step.
					(currentCharacter between: Character tab and: Character cr) or: [ currentCharacter = Character space ] ] whileTrue.
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $"
		ifTrue: [ 
			self recordMatch: #(34).
			self step.
			currentCharacter ~= $"
				ifTrue: [ 
					[ 
					self step.
					currentCharacter ~= $" ] whileTrue.
					currentCharacter = $"
						ifTrue: [ ^ self recordAndReportMatch: #comment ].
					^ self reportLastMatch ].
			currentCharacter = $"
				ifTrue: [ ^ self recordAndReportMatch: #comment ].
			^ self reportLastMatch ].
	currentCharacter = $#
		ifTrue: [ ^ self recordAndReportMatch: #(1 34) ].
	currentCharacter = $$
		ifTrue: [ 
			self recordMatch: #(34).
			self step.
			currentCharacter <= $
				ifTrue: [ ^ self recordAndReportMatch: #(30) ].
			^ self reportLastMatch ].
	currentCharacter = $&
		ifTrue: [ 
			self recordMatch: #(26 34).
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(33).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					^ self reportLastMatch ].
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			^ self reportLastMatch ].
	currentCharacter = $'
		ifTrue: [ 
			self recordMatch: #(34).
			self step.
			currentCharacter ~= $'
				ifTrue: [ ^ self scan2 ].
			currentCharacter = $'
				ifTrue: [ ^ self scan1 ].
			^ self reportLastMatch ].
	currentCharacter = $(
		ifTrue: [ ^ self recordAndReportMatch: #(10 34) ].
	currentCharacter = $)
		ifTrue: [ ^ self recordAndReportMatch: #(4 34) ].
	currentCharacter = $-
		ifTrue: [ 
			self recordMatch: #(26 34).
			self step.
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			(currentCharacter between: $0 and: $9)
				ifTrue: [ ^ self recordAndReportMatch: #negativenumber ].
			^ self reportLastMatch ].
	currentCharacter = $.
		ifTrue: [ ^ self recordAndReportMatch: #(5 34) ].
	currentCharacter = $:
		ifTrue: [ 
			self recordMatch: #(18 34).
			self step.
			currentCharacter = $=
				ifTrue: [ ^ self recordAndReportMatch: #(13) ].
			^ self reportLastMatch ].
	currentCharacter = $;
		ifTrue: [ ^ self recordAndReportMatch: #(2 34) ].
	currentCharacter = $<
		ifTrue: [ 
			self recordMatch: #(6 26 34).
			self step.
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			^ self reportLastMatch ].
	currentCharacter = $>
		ifTrue: [ 
			self recordMatch: #(7 26 34).
			self step.
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			^ self reportLastMatch ].
	currentCharacter = $?
		ifTrue: [ 
			self recordMatch: #(26 34).
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(32).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					^ self reportLastMatch ].
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			^ self reportLastMatch ].
	currentCharacter = $[
		ifTrue: [ ^ self recordAndReportMatch: #(17 34) ].
	currentCharacter = $]
		ifTrue: [ ^ self recordAndReportMatch: #(19 34) ].
	currentCharacter = $^
		ifTrue: [ ^ self recordAndReportMatch: #(16 34) ].
	currentCharacter = ${
		ifTrue: [ ^ self recordAndReportMatch: #(3 34) ].
	currentCharacter = $|
		ifTrue: [ 
			self recordMatch: #(9 26 34).
			self step.
			(currentCharacter = $!!
				or: [ 
					(currentCharacter between: $% and: $&)
						or: [ 
							(currentCharacter between: $* and: $,)
								or: [ 
									currentCharacter = $/
										or: [ 
											(currentCharacter between: $< and: $@)
												or: [ currentCharacter = $\ or: [ currentCharacter = $| or: [ currentCharacter = $~ ] ] ] ] ] ] ])
				ifTrue: [ ^ self recordAndReportMatch: #(26) ].
			^ self reportLastMatch ].
	currentCharacter = $}
		ifTrue: [ ^ self recordAndReportMatch: #(8 34) ].
	^ self reportLastMatch! !

!SmalltalkTermScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
scanNumber: id
	
	| token number |
	number := Number readFrom: stream.
	outputStream reset.
	token := SmaCCToken
		value: number
		start: start
		id: (Array with: id).
	matchActions := nil.
	returnMatchBlock value: token! !

!SmalltalkTermScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
variableassignment
	
	| token |
	outputStream skip: -2.
	stream skip: -2.
	token := SmaCCToken
		value: outputStream contents
		start: start
		id: (Array with: self nameId).
	outputStream reset.
	matchActions := nil.
	returnMatchBlock value: token! !

SmaCCScanner subclass: #SoulScanner
	instanceVariableNames: 'commentText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulScanner class methodsFor: 'generated-initialization' stamp: 'AndyKellens 10/28/2011 15:22'!
initializeKeywordMap
	keywordMap := Dictionary new.
	#(#(18 'if' 7) #(18 'tabled' 3) #(19 'matches:' 17))
		do: [ :each | (keywordMap at: each first ifAbsentPut: [ Dictionary new ]) at: (each at: 2) put: each last ].
	^ keywordMap! !

!SoulScanner class methodsFor: 'generated-comments' stamp: 'AndyKellens 10/28/2011 15:22'!
scannerDefinitionComment

	"<id> : [A-Za-z0-9] ([A-Za-z0-9])*;
<keyword> : [A-Za-z0-9] ([A-Za-z0-9])* \:;
<symbol> : \# [A-Za-z0-9\:]+;
<varid> : \? ([A-Za-z0-9])+;
<varkeyword> : \? ([A-Za-z0-9])+ \:;
<delayedvar> : \& ([A-Za-z0-9])+;
<posvar> : \+\? ([A-Za-z0-9])+;
<negvar> : \-\? ([A-Za-z0-9])+;
<underscorevar> : \?;
<stterm> : \[;
<quoted> : \{;
<dcgoperator> : \-\-\>;
<comment>	:	\"" [^\""]* \""	;
<whitespace> : [\s\t\r\n]+ ;

	
	"! !

!SoulScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
comment
	
	| newComment |
	newComment := outputStream contents copyReplaceAll: '""' with: '"'.
	newComment := newComment size > 2
		ifTrue:
			[newComment
				copyFrom: 2
				to: newComment size - 1]
		ifFalse: [''].
	commentText := (commentText
		ifNil: ['']
		ifNotNil:
			[commentText , (String with: Character cr) , (String with: Character cr)])
		, newComment.
	^super comment! !

!SoulScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
emptySymbolTokenId
	^65! !

!SoulScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
errorTokenId
	^66! !

!SoulScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
giveAndWipeComment
	
	| hold |
	hold := commentText.
	commentText := nil.
	^hold! !

!SoulScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
innerScanUpTo: stopChar ignore: startChar
	
	| level |
	level := 0.
	[currentCharacter = stopChar and: [level = 0]]
		whileFalse:
			[currentCharacter = startChar ifTrue: [level := level + 1].
			currentCharacter = stopChar ifTrue: [level := level - 1].
			self atEnd
				ifTrue: [self error: 'Unmatched ' , startChar asString]
				ifFalse: [self step]]! !

!SoulScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
quoted
	
	| token |
	self scanUpTo: $} ignore: ${.
	token := SmaCCToken
		value: outputStream contents
		start: start
		id: (Array with: self quotedId).
	matchActions := nil.
	returnMatchBlock value: token! !

!SoulScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
quotedId
	^28! !

!SoulScanner methodsFor: 'generated-scanner' stamp: 'AndyKellens 10/28/2011 15:22'!
scanForToken
	self step.
	((currentCharacter between: $0 and: $9)
		or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
		ifTrue: [ 
			[ 
			self recordMatch: #(18).
			self step.
			(currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
			currentCharacter = $:
				ifTrue: [ ^ self recordAndReportMatch: #(19) ].
			^ self reportLastMatch ].
	((currentCharacter between: Character tab and: Character cr) or: [ currentCharacter = Character space ])
		ifTrue: [ 
			[ 
			self recordMatch: #whitespace.
			self step.
			(currentCharacter between: Character tab and: Character cr) or: [ currentCharacter = Character space ] ] whileTrue.
			^ self reportLastMatch ].
	currentCharacter = $!!
		ifTrue: [ ^ self recordAndReportMatch: #(13) ].
	currentCharacter = $"
		ifTrue: [ 
			[ 
			self step.
			currentCharacter ~= $" ] whileTrue.
			currentCharacter = $"
				ifTrue: [ ^ self recordAndReportMatch: #comment ].
			^ self reportLastMatch ].
	currentCharacter = $#
		ifTrue: [ 
			self step.
			((currentCharacter between: $0 and: $:)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(20).
					self step.
					(currentCharacter between: $0 and: $:)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $&
		ifTrue: [ 
			self recordMatch: #(1).
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(23).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $(
		ifTrue: [ ^ self recordAndReportMatch: #(2) ].
	currentCharacter = $)
		ifTrue: [ ^ self recordAndReportMatch: #(5) ].
	currentCharacter = $+
		ifTrue: [ 
			self step.
			currentCharacter = $?
				ifTrue: [ 
					self step.
					((currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
						ifTrue: [ 
							[ 
							self recordMatch: #(24).
							self step.
							(currentCharacter between: $0 and: $9)
								or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
							^ self reportLastMatch ].
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $,
		ifTrue: [ ^ self recordAndReportMatch: #(4) ].
	currentCharacter = $-
		ifTrue: [ 
			self step.
			currentCharacter = $-
				ifTrue: [ 
					self step.
					currentCharacter = $>
						ifTrue: [ ^ self recordAndReportMatch: #(29) ].
					^ self reportLastMatch ].
			currentCharacter = $>
				ifTrue: [ ^ self recordAndReportMatch: #(16) ].
			currentCharacter = $?
				ifTrue: [ 
					self step.
					((currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
						ifTrue: [ 
							[ 
							self recordMatch: #(25).
							self step.
							(currentCharacter between: $0 and: $9)
								or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
							^ self reportLastMatch ].
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $.
		ifTrue: [ ^ self recordAndReportMatch: #(9) ].
	currentCharacter = $:
		ifTrue: [ ^ self recordAndReportMatch: #(6) ].
	currentCharacter = $<
		ifTrue: [ 
			self recordMatch: #(14).
			self step.
			currentCharacter = $>
				ifTrue: [ ^ self recordAndReportMatch: #(10) ].
			^ self reportLastMatch ].
	currentCharacter = $=
		ifTrue: [ ^ self recordAndReportMatch: #(8) ].
	currentCharacter = $>
		ifTrue: [ ^ self recordAndReportMatch: #(12) ].
	currentCharacter = $?
		ifTrue: [ 
			self recordMatch: #(26).
			self step.
			((currentCharacter between: $0 and: $9)
				or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ])
				ifTrue: [ 
					[ 
					self recordMatch: #(21).
					self step.
					(currentCharacter between: $0 and: $9)
						or: [ (currentCharacter between: $A and: $Z) or: [ currentCharacter between: $a and: $z ] ] ] whileTrue.
					currentCharacter = $:
						ifTrue: [ ^ self recordAndReportMatch: #(22) ].
					^ self reportLastMatch ].
			^ self reportLastMatch ].
	currentCharacter = $@
		ifTrue: [ 
			self step.
			currentCharacter = $(
				ifTrue: [ ^ self recordAndReportMatch: #(15) ].
			^ self reportLastMatch ].
	currentCharacter = $[
		ifTrue: [ ^ self recordAndReportMatch: #stterm ].
	currentCharacter = ${
		ifTrue: [ ^ self recordAndReportMatch: #quoted ].
	currentCharacter = $|
		ifTrue: [ ^ self recordAndReportMatch: #(11) ].
	^ self reportLastMatch! !

!SoulScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
scanUpTo: stopChar ignore: startChar
	
	| level |
	level := 0.
	self atEnd ifTrue: [self error: 'Unmatched ' , startChar asString].
	self step.
	[currentCharacter = stopChar and: [level = 0]]
		whileFalse:
			[currentCharacter = startChar ifTrue: [level := level + 1].
			currentCharacter = stopChar ifTrue: [level := level - 1].
			currentCharacter = $# ifTrue: [self step].
			currentCharacter = $'
				ifTrue:
					[self step.
					self innerScanUpTo: $' ignore: nil].
			currentCharacter = $$ ifTrue: [self step].
			self atEnd
				ifTrue: [self error: 'Unmatched ' , startChar asString]
				ifFalse: [self step]]	"self getNextChar."
	"self token: outputStream contents"! !

!SoulScanner methodsFor: 'scanning' stamp: 'sp 24/10/11 15:26'!
stterm
	
	| token |
	self scanUpTo: $] ignore: $[.
	token := SmaCCToken
		value: outputStream contents
		start: start
		id: (Array with: self sttermId).
	matchActions := nil.
	returnMatchBlock value: token! !

!SoulScanner methodsFor: 'generated-tokens' stamp: 'AndyKellens 10/28/2011 15:22'!
sttermId
	^27! !

RBPatternVariableNode subclass: #SoulRBPatternVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulRBPatternVariableNode methodsFor: 'initialize-release' stamp: 'sp 24/10/11 15:26'!
initializePatternVariables
	
	| name |
	name := self name.
	isAnything := isList := isLiteral := isStatement := recurseInto := false.
	2
		to: name size
		do:
			[:i | 
			| character |
			character := name at: i.
			character == self listCharacter
				ifTrue: [isAnything := isList := true]
				ifFalse:
					[character == self literalCharacter
						ifTrue: [isLiteral := true]
						ifFalse:
							[character == self statementCharacter
								ifTrue: [isStatement := true]
								ifFalse:
									[character == self recurseIntoCharacter
										ifTrue: [recurseInto := true]
										ifFalse: [^self]]]]]! !

SmaCCParser subclass: #QuotedCodeParser
	instanceVariableNames: 'builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!QuotedCodeParser class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	
	^super new initialize! !

!QuotedCodeParser class methodsFor: 'generated-comments' stamp: 'AndyKellens 10/28/2011 15:22'!
parserDefinitionComment

	"%id <compound>;

QUOTEDCODE : 	QUOTEDCODEPART*;

QUOTEDCODEPART :	STRINGPART {#liftFirstValue:} |
						VARIABLE {#liftFirstValue:} |
						SPLICECOMPOUND {#liftFirstValue:} ;

STRINGPART : <char> 'char'	{builder buildQuotedCodeConstant: char value} |
				<space> 'space' {builder buildQuotedCodeConstant: space value}|
                        <quote> 'quote' {builder buildQuotedCodeConstant: '''',quote value};

VARIABLE :	NORMALVAR {#liftFirstValue:} | DECORATEDVAR {#liftFirstValue:};

NORMALVAR :  <var> 'var' 	{builder buildVariable: var value};
DECORATEDVAR : <var> 'var' TYPE+ 'types' {builder buildDecoratedVariable: var value with: types};

TYPE : <type> 'name' {builder buildConstant: (name value allButFirst: 1)};

SPLICECOMPOUND : <compound> 'compound' {builder buildSpliceCompound: compound value};
	
	"! !

!QuotedCodeParser class methodsFor: 'generated-accessing' stamp: 'AndyKellens 10/28/2011 15:22'!
scannerClass
	^QuotedCodeScanner! !

!QuotedCodeParser class methodsFor: 'generated-starting states' stamp: 'AndyKellens 10/28/2011 15:22'!
startingStateForQUOTEDCODE
	^1! !

!QuotedCodeParser methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
initialize
	
	builder := SoulParseTreeBuilder forParser: self.
	^super initialize! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForDECORATEDVAR1: nodes
	^ builder buildDecoratedVariable: (nodes at: 1) value with: (nodes at: 2)! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForNORMALVAR1: nodes
	^ builder buildVariable: (nodes at: 1) value! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForRepeatXMultipleXXTYPE1: nodes
	^ OrderedCollection with: (nodes at: 1)! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForRepeatXMultipleXXTYPE2: nodes
	^ (nodes at: 1)
		add: (nodes at: 2);
		yourself! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForRepeatXXQUOTEDCODEPART1: nodes
	^ OrderedCollection new! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForRepeatXXQUOTEDCODEPART2: nodes
	^ (nodes at: 1)
		add: (nodes at: 2);
		yourself! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForSPLICECOMPOUND1: nodes
	^ builder buildSpliceCompound: (nodes at: 1) value! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForSTRINGPART1: nodes
	^ builder buildQuotedCodeConstant: (nodes at: 1) value! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForSTRINGPART2: nodes
	^ builder buildQuotedCodeConstant: (nodes at: 1) value! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForSTRINGPART3: nodes
	^ builder buildQuotedCodeConstant: '''' , (nodes at: 1) value! !

!QuotedCodeParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTYPE1: nodes
	^ builder buildConstant: ((nodes at: 1) value allButFirst: 1)! !

!QuotedCodeParser methodsFor: 'generated-tables' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceTable
	^#(
#(7 1 #liftFirstValue:)
#(7 1 #liftFirstValue:)
#(8 1 #reduceActionForSPLICECOMPOUND1:)
#(9 1 #reduceActionForSTRINGPART1:)
#(9 1 #reduceActionForSTRINGPART2:)
#(9 1 #reduceActionForSTRINGPART3:)
#(10 1 #reduceFor:)
#(11 1 #reduceFor:)
#(12 1 #reduceActionForNORMALVAR1:)
#(13 2 #reduceActionForDECORATEDVAR1:)
#(14 0 #reduceActionForRepeatXXQUOTEDCODEPART1:)
#(14 2 #reduceActionForRepeatXXQUOTEDCODEPART2:)
#(15 1 #reduceActionForTYPE1:)
#(16 1 #reduceActionForRepeatXMultipleXXTYPE1:)
#(16 2 #reduceActionForRepeatXMultipleXXTYPE2:)
#(17 1 #liftFirstValue:)
#(17 1 #liftFirstValue:)
#(17 1 #liftFirstValue:)
)! !

!QuotedCodeParser methodsFor: 'generated-tables' stamp: 'AndyKellens 10/28/2011 15:22'!
transitionTable
	^#(
#(3 46 1 46 2 46 3 46 5 46 6 9 11 13 14 46 18)
#(2 0 18)
#(3 17 1 21 2 25 3 29 5 33 6 37 7 41 8 45 9 49 12 53 13 57 17 34 18)
#(2 18 1 2 3 5 6 18)
#(2 22 1 2 3 5 6 18)
#(3 38 1 38 2 38 3 61 4 38 5 38 6 65 15 69 16 38 18)
#(2 26 1 2 3 5 6 18)
#(2 14 1 2 3 5 6 18)
#(2 70 1 2 3 5 6 18)
#(2 74 1 2 3 5 6 18)
#(2 66 1 2 3 5 6 18)
#(2 6 1 2 3 5 6 18)
#(2 10 1 2 3 5 6 18)
#(2 50 1 2 3 5 6 18)
#(2 54 1 2 3 4 5 6 18)
#(2 58 1 2 3 4 5 6 18)
#(3 42 1 42 2 42 3 61 4 42 5 42 6 73 15 42 18)
#(2 62 1 2 3 4 5 6 18)
)! !

SmaCCParser subclass: #SmalltalkTermParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SmalltalkTermParser class methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
parseExpression: aString onError: aBlock
	
	^self
		parse: aString
		startingAt: self startingStateForSequenceNode
		onError: aBlock! !

!SmalltalkTermParser class methodsFor: 'generated-comments' stamp: 'AndyKellens 10/28/2011 15:22'!
parserDefinitionComment

	"%id <number> <negativenumber> <name> <binarysymbol>;
%start SequenceNode ;

Method:
	  <name> MethodSequenceNode {(RBMethodNode selector: '1' value asSymbol arguments: #() body: '2' last) tags: '2' first; yourself}
	| BinaryMethodName MethodSequenceNode {(RBMethodNode selector: '1' first arguments: '1' last body: '2' last) tags: '2' first; yourself}
	| KeywordMethodName MethodSequenceNode {(RBMethodNode selector: '1' first contents asSymbol arguments: '1' last body: '2' last) tags: '2' first; yourself};

BinaryMethodName:
	<binarysymbol> Variable {Array with: '1' value asSymbol with: (Array with: '2')} ;

KeywordMethodName:
	<keyword> Variable {| stream | stream := WriteStream on: (String new: 30). stream nextPutAll: '1' value. Array with: stream with: (OrderedCollection with: '2')}
	| KeywordMethodName <keyword> Variable {'1' first nextPutAll: '2' value. '1' last add: '3'. '1'};

Primitives:
	{OrderedCollection new}
	| Primitives ""<"" PrimitiveMessageSend "">"" {'1' add: ('2' startPosition to: '4' startPosition); yourself};

PrimitiveMessageSend:
	| PrimitiveMessageSend <keyword> {nil}
	| PrimitiveMessageSend Literal {nil}
	| PrimitiveMessageSend ""("" {nil}
	| PrimitiveMessageSend "")"" {nil}
	| PrimitiveMessageSend ""{"" {nil}
	| PrimitiveMessageSend ""}"" {nil}
	| PrimitiveMessageSend "";"" {nil}
	| PrimitiveMessageSend ""."" {nil}
	| PrimitiveMessageSend <name> {nil}
	| PrimitiveMessageSend <binarysymbol> {nil};

MethodSequenceNode:
	Primitives ""|"" TemporaryVariables ""|"" Primitives Statements {Array with: ('1' addAll: '5'; yourself) with: (RBSequenceNode temporaries: '3' statements: '6')}
	| Primitives ""||"" Primitives Statements {Array with: ('1' addAll: '3'; yourself) with: (RBSequenceNode temporaries: #() statements: '4')}
	| Primitives Statements {Array with: '1' with: (RBSequenceNode temporaries: #() statements: '2')};

SequenceNode:
	Temporaries Statements {RBSequenceNode temporaries: '1' statements: '2'};

Temporaries:
	{#()}
	| ""||"" {#()}
	| ""|"" TemporaryVariables ""|"" {'2'};

TemporaryVariables:
	{OrderedCollection new}
	| TemporaryVariables Variable {'1' add: '2'; yourself};

Statements:
	{OrderedCollection new}
	| StatementList "".""? {#liftFirstValue:}
	| StatementList ""."" ""^"" Expression "".""? {'1' add: (RBReturnNode value: '4'); yourself}
	| ""^"" Expression "".""? {OrderedCollection with: (RBReturnNode value: '2')};

StatementList:
	Expression
	| StatementList ""."" Expression {'1' add: '3'; yourself};

Block:
	""["" BlockArgs ""|"" SequenceNode ""]"" {RBBlockNode arguments: '2' body: '4'}
	| ""["" SequenceNode ""]"" {RBBlockNode body: '2'}
	| ""["" BlockArgs ""]"" {RBBlockNode arguments: '2' body: (RBSequenceNode statements: #())}
	| ""["" BlockArgs ""||"" TemporaryVariables ""|"" Statements ""]"" {RBBlockNode arguments: '2' body: (RBSequenceNode temporaries: '4' statements: '6')};

BlockArgs:
	"":"" Variable {OrderedCollection with: '2'}
	| BlockArgs "":"" Variable {'1' add: '3'; yourself};


Expression:
	Variable "":="" Expression {RBAssignmentNode variable: '1' value: '3'}
|	Cascade {#liftFirstValue:};

Cascade:
	MessageSend CascadeList 
				{| messages |
				'2' isEmpty ifTrue: [^'1'].
				messages := OrderedCollection new: '2' size + 1.
				messages add: '1'.
				'2' do: [:each | messages add: (RBMessageNode receiver: '1' receiver selector: each first contents asSymbol arguments: each last)].
				RBCascadeNode messages: messages}
|	Primary {#liftFirstValue:};

MessageSend:
	KeywordMessageSend {#liftFirstValue:}
|	BinaryMessageSend {#liftFirstValue:}
|	UnaryMessageSend {#liftFirstValue:};

CascadeList:
	{OrderedCollection new}
|	CascadeList "";"" SimpleMessage {'1' add: '3'; yourself};

SimpleMessage:
	UnaryMessage {#liftFirstValue:}
|	BinaryMessage {#liftFirstValue:}
|	KeywordMessage {#liftFirstValue:};

KeywordMessageSend:
	BinaryMessageSend KeywordMessage {#createMessageNodeFrom:}
|	UnaryMessageSend KeywordMessage {#createMessageNodeFrom:}
|	Primary KeywordMessage {#createMessageNodeFrom:};

KeywordMessage:
	<keyword> KeywordArgument {| stream | stream := WriteStream on: (String new: 30). stream nextPutAll: '1' value. Array with: stream with: (OrderedCollection with: '2')}
|	KeywordMessage <keyword> KeywordArgument {'1' first nextPutAll: '2' value. '1' last add: '3'. '1'};

KeywordArgument:
	BinaryMessageSend {#liftFirstValue:}
|	UnaryMessageSend {#liftFirstValue:}
|	Primary {#liftFirstValue:};

BinaryMessageSend:
	BinaryMessageSend BinaryMessage {#createMessageNodeFrom:}
|	UnaryMessageSend BinaryMessage {#createMessageNodeFrom:}
|	Primary BinaryMessage {#createMessageNodeFrom:} ;

BinaryMessage :
	<binarysymbol> BinaryArgument {Array with: (ReadStream on: '1' value) with: (Array with: '2')};

BinaryArgument:
	UnaryMessageSend {#liftFirstValue:}
|	Primary {#liftFirstValue:};

UnaryMessageSend :
	UnaryMessageSend UnaryMessage {#createMessageNodeFrom:}
|	Primary UnaryMessage {#createMessageNodeFrom:};

UnaryMessage :
	<name> {Array with: (ReadStream on: '1' value) with: #()};


Primary: 
	""("" Expression "")"" {#liftSecondValue:}
|	Literal {RBLiteralNode value: '1'}
|	Variable {#liftFirstValue:}
|	Block {#liftFirstValue:};

Variable: 
	<name> {RBVariableNode named: '1' value}
|	<soulvar> {RBSoulVariableNode named: ('1' value allButFirst: 1)}
|     <delayedsoulvar> {RBDelayedSoulVariableNode named: ('1' value allButFirst: 1)}
|    <dottedname> {RBVariableNode named: '1' value};

Literal:
	""true"" {true}
|	""false"" {false}
|	""nil"" {nil}
|	<number> {'1' value}
|	<negativenumber> {'1' value}
|	<character> {'1' value at: 2}
|	<string> {('1' value copyFrom: 2 to: '1' value size - 1) copyReplaceAll: '''''' with: ''''}
|	 ""#"" <string> {(('2' value copyFrom: 2 to: '2' value size - 1) copyReplaceAll: '''''' with: '''') asSymbol}
|	""#"" <name> {'2' value asSymbol}
|	""#"" <binarysymbol> {'2' value asSymbol}
|	""#"" <keyword> {'2' value asSymbol}
|	""#"" <multikeyword> {'2' value asSymbol}
|	""#"" ""["" ByteArray ""]"" {'3' contents}
|	""#"" ""("" Array "")"" {'3' contents}
|	""#"" ""{"" <name> ""}"" {'3' value asQualifiedReference}
|	""#"" ""{"" <dottedname> ""}"" {'3' value asQualifiedReference};

ByteArray: 
	{WriteStream on: ByteArray new}
|	ByteArray <number> {'1' nextPut: '2' value; yourself};

Array: 
	{WriteStream on: Array new}
|	Array ArrayLiteral {'1' nextPut: '2'; yourself} 
|	Array <name> "":="" {'2' stopPosition + 1 = '3' startPosition ifTrue: ['1' nextPut: ('2' value, ':') asSymbol] ifFalse: ['1' nextPut: '2' value asSymbol; nextPut: #':']. scanner position: '3' startPosition. currentToken := nil. '1'};

ArrayLiteral:
	Literal {#liftFirstValue:}
|	<name> {'1' value asSymbol}
|	<binarysymbol> {'1' value asSymbol}
|	<keyword> {'1' value asSymbol}
|	<multikeyword> {'1' value asSymbol}
|	""{"" <name> ""}"" {'2' value asQualifiedReference}
|	""{"" <dottedname> ""}"" {'2' value asQualifiedReference}
|	""("" Array "")"" {'2' contents}
|	""["" ByteArray ""]"" {'2' contents}
|	<anychar> {'1' value asSymbol}
|	<dottedname> {'1' value};
	
	"! !

!SmalltalkTermParser class methodsFor: 'generated-accessing' stamp: 'AndyKellens 10/28/2011 15:22'!
scannerClass
	^SmalltalkTermScanner! !

!SmalltalkTermParser class methodsFor: 'generated-starting states' stamp: 'AndyKellens 10/28/2011 15:22'!
startingStateForMethod
	^1! !

!SmalltalkTermParser class methodsFor: 'generated-starting states' stamp: 'AndyKellens 10/28/2011 15:22'!
startingStateForSequenceNode
	^2! !

!SmalltalkTermParser methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
actionForCurrentToken
	
	| ids action |
	ids := currentToken id.
	1
		to: ids size
		do:
			[:i | 
			action := self actionFor: (ids at: i).
			(action bitAnd: self actionMask) = self errorAction ifFalse: [^action].
			(ids at: i) = scanner negativenumberId
				ifTrue:
					[currentToken := SmaCCToken
						value: '-'
						start: currentToken startPosition
						id: (Array with: scanner binarysymbolId).
					scanner position: currentToken stopPosition - 1.
					^self actionForCurrentToken]].
	^self errorAction! !

!SmalltalkTermParser methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
createMessageNodeFrom: aCollection
	
	^RBMessageNode
		receiver: aCollection first
		selector: (aCollection at: 2) first contents asSymbol
		arguments: (aCollection at: 2) last! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArray1: nodes
	^ WriteStream on: Array new! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArray2: nodes
	^ (nodes at: 1)
		nextPut: (nodes at: 2);
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArray3: nodes
	(nodes at: 2) stopPosition + 1 = (nodes at: 3) startPosition
		ifTrue: [ (nodes at: 1) nextPut: ((nodes at: 2) value , ':') asSymbol ]
		ifFalse: [ 
			(nodes at: 1)
				nextPut: (nodes at: 2) value asSymbol;
				nextPut: #':' ].
	scanner position: (nodes at: 3) startPosition.
	currentToken := nil.
	^ nodes at: 1! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral10: nodes
	^ (nodes at: 1) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral11: nodes
	^ (nodes at: 1) value! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral2: nodes
	^ (nodes at: 1) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral3: nodes
	^ (nodes at: 1) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral4: nodes
	^ (nodes at: 1) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral5: nodes
	^ (nodes at: 1) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral6: nodes
	^ (nodes at: 2) value asQualifiedReference! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral7: nodes
	^ (nodes at: 2) value asQualifiedReference! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral8: nodes
	^ (nodes at: 2) contents! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForArrayLiteral9: nodes
	^ (nodes at: 2) contents! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBinaryMessage1: nodes
	^ Array with: (ReadStream on: (nodes at: 1) value) with: (Array with: (nodes at: 2))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBinaryMethodName1: nodes
	^ Array with: (nodes at: 1) value asSymbol with: (Array with: (nodes at: 2))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBlock1: nodes
	^ RBBlockNode arguments: (nodes at: 2) body: (nodes at: 4)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBlock2: nodes
	^ RBBlockNode body: (nodes at: 2)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBlock3: nodes
	^ RBBlockNode arguments: (nodes at: 2) body: (RBSequenceNode statements: #())! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBlock4: nodes
	^ RBBlockNode arguments: (nodes at: 2) body: (RBSequenceNode temporaries: (nodes at: 4) statements: (nodes at: 6))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBlockArgs1: nodes
	^ OrderedCollection with: (nodes at: 2)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForBlockArgs2: nodes
	^ (nodes at: 1)
		add: (nodes at: 3);
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForByteArray1: nodes
	^ WriteStream on: ByteArray new! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForByteArray2: nodes
	^ (nodes at: 1)
		nextPut: (nodes at: 2) value;
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCascade1: nodes
	| messages |
	(nodes at: 2) isEmpty
		ifTrue: [ ^ nodes at: 1 ].
	messages := OrderedCollection new: (nodes at: 2) size + 1.
	messages add: (nodes at: 1).
	(nodes at: 2)
		do: [ :each | 
			messages
				add: (RBMessageNode receiver: (nodes at: 1) receiver selector: each first contents asSymbol arguments: each last) ].
	^ RBCascadeNode messages: messages! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCascadeList1: nodes
	^ OrderedCollection new! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCascadeList2: nodes
	^ (nodes at: 1)
		add: (nodes at: 3);
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForExpression1: nodes
	^ RBAssignmentNode variable: (nodes at: 1) value: (nodes at: 3)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKeywordMessage1: nodes
	| stream |
	stream := WriteStream on: (String new: 30).
	stream nextPutAll: (nodes at: 1) value.
	^ Array with: stream with: (OrderedCollection with: (nodes at: 2))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKeywordMessage2: nodes
	(nodes at: 1) first nextPutAll: (nodes at: 2) value.
	(nodes at: 1) last add: (nodes at: 3).
	^ nodes at: 1! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKeywordMethodName1: nodes
	| stream |
	stream := WriteStream on: (String new: 30).
	stream nextPutAll: (nodes at: 1) value.
	^ Array with: stream with: (OrderedCollection with: (nodes at: 2))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKeywordMethodName2: nodes
	(nodes at: 1) first nextPutAll: (nodes at: 2) value.
	(nodes at: 1) last add: (nodes at: 3).
	^ nodes at: 1! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral10: nodes
	^ (nodes at: 2) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral11: nodes
	^ (nodes at: 2) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral12: nodes
	^ (nodes at: 2) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral13: nodes
	^ (nodes at: 3) contents! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral14: nodes
	^ (nodes at: 3) contents! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral15: nodes
	^ (nodes at: 3) value asQualifiedReference! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral16: nodes
	^ (nodes at: 3) value asQualifiedReference! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral1: nodes
	^ true! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral2: nodes
	^ false! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral3: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral4: nodes
	^ (nodes at: 1) value! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral5: nodes
	^ (nodes at: 1) value! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral6: nodes
	^ (nodes at: 1) value at: 2! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral7: nodes
	^ ((nodes at: 1) value copyFrom: 2 to: (nodes at: 1) value size - 1) copyReplaceAll: '''''' with: ''''! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral8: nodes
	^ (((nodes at: 2) value copyFrom: 2 to: (nodes at: 2) value size - 1) copyReplaceAll: '''''' with: '''') asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLiteral9: nodes
	^ (nodes at: 2) value asSymbol! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMethod1: nodes
	^ (RBMethodNode selector: (nodes at: 1) value asSymbol arguments: #() body: (nodes at: 2) last)
		tags: (nodes at: 2) first;
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMethod2: nodes
	^ (RBMethodNode selector: (nodes at: 1) first arguments: (nodes at: 1) last body: (nodes at: 2) last)
		tags: (nodes at: 2) first;
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMethod3: nodes
	^ (RBMethodNode selector: (nodes at: 1) first contents asSymbol arguments: (nodes at: 1) last body: (nodes at: 2) last)
		tags: (nodes at: 2) first;
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMethodSequenceNode1: nodes
	^ Array
		with:
			((nodes at: 1)
				addAll: (nodes at: 5);
				yourself)
		with: (RBSequenceNode temporaries: (nodes at: 3) statements: (nodes at: 6))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMethodSequenceNode2: nodes
	^ Array
		with:
			((nodes at: 1)
				addAll: (nodes at: 3);
				yourself)
		with: (RBSequenceNode temporaries: #() statements: (nodes at: 4))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMethodSequenceNode3: nodes
	^ Array with: (nodes at: 1) with: (RBSequenceNode temporaries: #() statements: (nodes at: 2))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForOptionalXXXXX1: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForOptionalXXXXX2: nodes
	^ nodes at: 1! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimary2: nodes
	^ RBLiteralNode value: (nodes at: 1)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend10: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend11: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend2: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend3: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend4: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend5: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend6: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend7: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend8: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitiveMessageSend9: nodes
	^ nil! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitives1: nodes
	^ OrderedCollection new! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPrimitives2: nodes
	^ (nodes at: 1)
		add: ((nodes at: 2) startPosition to: (nodes at: 4) startPosition);
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForSequenceNode1: nodes
	^ RBSequenceNode temporaries: (nodes at: 1) statements: (nodes at: 2)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForStatementList2: nodes
	^ (nodes at: 1)
		add: (nodes at: 3);
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForStatements1: nodes
	^ OrderedCollection new! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForStatements3: nodes
	^ (nodes at: 1)
		add: (RBReturnNode value: (nodes at: 4));
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForStatements4: nodes
	^ OrderedCollection with: (RBReturnNode value: (nodes at: 2))! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTemporaries1: nodes
	^ #()! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTemporaries2: nodes
	^ #()! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTemporaries3: nodes
	^ nodes at: 2! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTemporaryVariables1: nodes
	^ OrderedCollection new! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTemporaryVariables2: nodes
	^ (nodes at: 1)
		add: (nodes at: 2);
		yourself! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForUnaryMessage1: nodes
	^ Array with: (ReadStream on: (nodes at: 1) value) with: #()! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVariable1: nodes
	^ RBVariableNode named: (nodes at: 1) value! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVariable2: nodes
	^ RBSoulVariableNode named: ((nodes at: 1) value allButFirst: 1)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVariable3: nodes
	^ RBDelayedSoulVariableNode named: ((nodes at: 1) value allButFirst: 1)! !

!SmalltalkTermParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVariable4: nodes
	^ RBVariableNode named: (nodes at: 1) value! !

!SmalltalkTermParser methodsFor: 'generated-tables' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceTable
	^#(
#(35 2 #reduceActionForKeywordMethodName1:)
#(35 3 #reduceActionForKeywordMethodName2:)
#(36 0 #reduceActionForStatements1:)
#(36 2 #liftFirstValue:)
#(36 5 #reduceActionForStatements3:)
#(36 3 #reduceActionForStatements4:)
#(37 0 #reduceActionForTemporaries1:)
#(37 1 #reduceActionForTemporaries2:)
#(37 3 #reduceActionForTemporaries3:)
#(38 1 #reduceFor:)
#(38 3 #reduceActionForStatementList2:)
#(39 0 #reduceActionForOptionalXXXXX1:)
#(39 1 #reduceActionForOptionalXXXXX2:)
#(40 3 #reduceActionForExpression1:)
#(40 1 #liftFirstValue:)
#(41 2 #reduceActionForBlockArgs1:)
#(41 3 #reduceActionForBlockArgs2:)
#(42 5 #reduceActionForBlock1:)
#(42 3 #reduceActionForBlock2:)
#(42 3 #reduceActionForBlock3:)
#(42 7 #reduceActionForBlock4:)
#(43 2 #reduceActionForCascade1:)
#(43 1 #liftFirstValue:)
#(44 1 #liftFirstValue:)
#(44 1 #liftFirstValue:)
#(44 1 #liftFirstValue:)
#(45 0 #reduceActionForCascadeList1:)
#(45 3 #reduceActionForCascadeList2:)
#(46 3 #liftSecondValue:)
#(46 1 #reduceActionForPrimary2:)
#(46 1 #liftFirstValue:)
#(46 1 #liftFirstValue:)
#(47 1 #reduceActionForLiteral1:)
#(47 1 #reduceActionForLiteral2:)
#(47 1 #reduceActionForLiteral3:)
#(47 1 #reduceActionForLiteral4:)
#(47 1 #reduceActionForLiteral5:)
#(47 1 #reduceActionForLiteral6:)
#(47 1 #reduceActionForLiteral7:)
#(47 2 #reduceActionForLiteral8:)
#(47 2 #reduceActionForLiteral9:)
#(47 2 #reduceActionForLiteral10:)
#(47 2 #reduceActionForLiteral11:)
#(47 2 #reduceActionForLiteral12:)
#(47 4 #reduceActionForLiteral13:)
#(47 4 #reduceActionForLiteral14:)
#(47 4 #reduceActionForLiteral15:)
#(47 4 #reduceActionForLiteral16:)
#(48 2 #createMessageNodeFrom:)
#(48 2 #createMessageNodeFrom:)
#(48 2 #createMessageNodeFrom:)
#(49 2 #createMessageNodeFrom:)
#(49 2 #createMessageNodeFrom:)
#(50 1 #liftFirstValue:)
#(50 1 #liftFirstValue:)
#(50 1 #liftFirstValue:)
#(51 1 #reduceActionForUnaryMessage1:)
#(52 2 #reduceActionForBinaryMessage1:)
#(53 2 #reduceActionForKeywordMessage1:)
#(53 3 #reduceActionForKeywordMessage2:)
#(54 1 #liftFirstValue:)
#(54 1 #liftFirstValue:)
#(54 1 #liftFirstValue:)
#(55 1 #liftFirstValue:)
#(55 1 #liftFirstValue:)
#(56 0 #reduceFor:)
#(56 2 #reduceActionForPrimitiveMessageSend2:)
#(56 2 #reduceActionForPrimitiveMessageSend3:)
#(56 2 #reduceActionForPrimitiveMessageSend4:)
#(56 2 #reduceActionForPrimitiveMessageSend5:)
#(56 2 #reduceActionForPrimitiveMessageSend6:)
#(56 2 #reduceActionForPrimitiveMessageSend7:)
#(56 2 #reduceActionForPrimitiveMessageSend8:)
#(56 2 #reduceActionForPrimitiveMessageSend9:)
#(56 2 #reduceActionForPrimitiveMessageSend10:)
#(56 2 #reduceActionForPrimitiveMessageSend11:)
#(57 0 #reduceActionForPrimitives1:)
#(57 4 #reduceActionForPrimitives2:)
#(58 2 #reduceActionForMethod1:)
#(58 2 #reduceActionForMethod2:)
#(58 2 #reduceActionForMethod3:)
#(59 1 #reduceActionForVariable1:)
#(59 1 #reduceActionForVariable2:)
#(59 1 #reduceActionForVariable3:)
#(59 1 #reduceActionForVariable4:)
#(60 2 #reduceActionForBinaryMethodName1:)
#(61 6 #reduceActionForMethodSequenceNode1:)
#(61 4 #reduceActionForMethodSequenceNode2:)
#(61 2 #reduceActionForMethodSequenceNode3:)
#(62 0 #reduceActionForByteArray1:)
#(62 2 #reduceActionForByteArray2:)
#(63 0 #reduceActionForArray1:)
#(63 2 #reduceActionForArray2:)
#(63 3 #reduceActionForArray3:)
#(64 1 #liftFirstValue:)
#(64 1 #reduceActionForArrayLiteral2:)
#(64 1 #reduceActionForArrayLiteral3:)
#(64 1 #reduceActionForArrayLiteral4:)
#(64 1 #reduceActionForArrayLiteral5:)
#(64 3 #reduceActionForArrayLiteral6:)
#(64 3 #reduceActionForArrayLiteral7:)
#(64 3 #reduceActionForArrayLiteral8:)
#(64 3 #reduceActionForArrayLiteral9:)
#(64 1 #reduceActionForArrayLiteral10:)
#(64 1 #reduceActionForArrayLiteral11:)
#(65 2 #createMessageNodeFrom:)
#(65 2 #createMessageNodeFrom:)
#(65 2 #createMessageNodeFrom:)
#(68 2 #reduceActionForSequenceNode1:)
#(69 1 #reduceFor:)
#(70 0 #reduceActionForTemporaryVariables1:)
#(70 2 #reduceActionForTemporaryVariables2:)
)! !

!SmalltalkTermParser methodsFor: 'generated-tables' stamp: 'AndyKellens 10/28/2011 15:22'!
transitionTable
	^#(
#(3 13 20 17 24 21 26 25 35 29 58 33 60)
#(3 30 1 37 9 30 10 30 11 30 12 30 14 41 15 30 16 30 17 30 20 30 21 30 22 30 23 30 29 30 30 30 32 30 33 45 37 30 66 49 68)
#(3 310 1 310 6 310 9 310 10 310 11 310 12 310 14 310 15 310 16 310 17 310 20 310 21 310 22 310 23 310 29 310 30 310 32 310 33 53 57 57 61 310 66)
#(3 61 20 65 29 69 32 73 33 77 59)
#(3 61 20 65 29 69 32 73 33 81 59)
#(3 310 1 310 6 310 9 310 10 310 11 310 12 310 14 310 15 310 16 310 17 310 20 310 21 310 22 310 23 85 24 310 29 310 30 310 32 310 33 53 57 89 61 310 66)
#(2 0 66)
#(3 310 1 310 6 310 9 310 10 310 11 310 12 310 14 310 15 310 16 310 17 310 20 310 21 310 22 310 23 310 29 310 30 310 32 310 33 53 57 93 61 310 66)
#(3 446 9 446 20 446 29 446 32 446 33 97 70)
#(2 34 1 10 11 12 14 16 17 19 20 21 22 23 29 30 32 33 66)
#(3 101 1 105 10 109 11 113 12 117 14 121 16 125 17 14 19 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 145 36 149 38 153 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65 14 66)
#(2 0 66)
#(3 101 1 193 6 197 9 105 10 109 11 113 12 117 14 201 15 121 16 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 205 36 149 38 153 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65 14 66)
#(2 318 66)
#(2 330 1 2 4 5 6 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 26 29 30 32 33 66)
#(2 342 1 2 4 5 6 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 26 29 30 32 33 66)
#(2 334 1 2 4 5 6 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 26 29 30 32 33 66)
#(2 338 1 2 4 5 6 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 26 29 30 32 33 66)
#(2 6 1 6 9 10 11 12 14 15 16 17 20 21 22 23 24 29 30 32 33 66)
#(2 346 1 6 9 10 11 12 14 15 16 17 20 21 22 23 29 30 32 33 66)
#(3 61 20 65 29 69 32 73 33 209 59)
#(2 326 66)
#(2 322 66)
#(3 213 9 61 20 65 29 69 32 73 33 217 59)
#(3 221 3 225 10 229 17 233 20 237 23 241 24 245 25 249 26)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 253 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65)
#(2 134 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 138 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 142 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 257 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65)
#(3 30 1 37 9 30 10 30 11 30 12 30 14 41 15 30 16 30 17 261 18 30 19 30 20 30 21 30 22 30 23 30 29 30 30 30 32 30 33 45 37 265 41 269 68)
#(2 146 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 150 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 158 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 154 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 438 19 66)
#(3 273 5 50 19 277 39 50 66)
#(2 42 5 19 66)
#(2 130 2 4 5 19 20 24 26 66)
#(2 62 4 5 19 66)
#(3 110 2 110 4 110 5 110 19 281 45 110 66)
#(3 94 4 94 5 94 19 285 20 289 24 293 26 297 51 301 52 305 53 94 66)
#(2 122 2 4 5 19 20 24 26 66)
#(3 102 2 102 4 102 5 102 19 289 24 293 26 309 52 313 53 102 66)
#(3 106 2 106 4 106 5 106 19 285 20 289 24 293 26 317 51 321 52 325 53 106 66)
#(3 126 4 126 5 329 13 126 19 126 20 126 24 126 26 126 66)
#(2 98 2 4 5 19 66)
#(3 266 1 266 2 266 3 266 4 266 5 266 7 266 8 266 10 266 11 266 12 266 14 266 20 266 21 266 22 266 23 266 24 266 26 266 30 333 56)
#(3 446 9 446 20 446 29 446 32 446 33 337 70)
#(3 310 1 310 6 310 10 310 11 310 12 310 14 310 16 310 17 310 20 310 21 310 22 310 23 310 29 310 30 310 32 310 33 341 57 310 66)
#(2 358 66)
#(2 10 1 6 9 10 11 12 14 15 16 17 20 21 22 23 24 29 30 32 33 66)
#(2 38 1 10 11 12 14 16 17 19 20 21 22 23 29 30 32 33 66)
#(2 450 9 20 29 32 33)
#(3 345 20 349 29)
#(3 370 1 370 3 370 4 370 10 370 11 370 12 370 14 370 17 370 20 370 21 370 22 370 23 370 24 370 25 370 26 370 29 370 30 370 34 353 63)
#(3 362 19 362 21 357 62)
#(2 166 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 162 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 174 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 178 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 170 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 361 4)
#(3 365 5 50 19 369 39 50 66)
#(3 61 20 65 29 69 32 73 33 373 59)
#(3 377 9 381 15 385 18 389 19)
#(2 393 19)
#(3 101 1 105 10 109 11 113 12 117 14 397 16 125 17 54 19 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 401 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65 54 66)
#(2 18 19 66)
#(3 405 2 90 4 90 5 90 19 90 66)
#(2 230 2 4 5 19 20 24 26 66)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 157 42 409 46 173 47 413 48 417 49 421 54 425 59)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 157 42 429 46 173 47 433 49 437 55 425 59)
#(2 214 2 4 5 19 20 24 26 66)
#(2 206 2 4 5 19 24 26 66)
#(3 434 2 434 4 434 5 434 19 441 24 434 66)
#(2 198 2 4 5 19 24 26 66)
#(3 426 2 426 4 426 5 426 19 441 24 426 66)
#(2 210 2 4 5 19 20 24 26 66)
#(2 202 2 4 5 19 24 26 66)
#(3 430 2 430 4 430 5 430 19 441 24 430 66)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 445 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65)
#(3 101 1 449 2 453 3 457 4 461 5 465 7 469 8 473 10 109 11 113 12 117 14 477 20 129 21 133 22 137 23 481 24 485 26 141 30 489 47)
#(3 493 9 61 20 65 29 69 32 73 33 217 59)
#(3 101 1 193 6 105 10 109 11 113 12 117 14 121 16 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 497 36 149 38 153 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65 14 66)
#(2 501 8)
#(2 505 8)
#(3 101 1 509 3 513 4 517 10 109 11 113 12 117 14 521 17 525 20 129 21 133 22 137 23 529 24 533 25 537 26 541 29 141 30 545 34 549 47 553 64)
#(3 557 19 561 21)
#(2 118 2 4 5 19 20 24 26 66)
#(2 54 19 66)
#(2 26 19 66)
#(2 66 9 15 18 19)
#(3 30 1 37 9 30 10 30 11 30 12 30 14 41 15 30 16 30 17 30 19 30 20 30 21 30 22 30 23 30 29 30 30 30 32 30 33 45 37 565 68)
#(3 446 9 446 20 446 29 446 32 446 33 569 70)
#(3 61 20 65 29 69 32 73 33 573 59)
#(2 82 2 4 5 19 20 24 26 66)
#(2 78 2 4 5 19 20 24 26 66)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 577 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65)
#(2 46 5 19 66)
#(3 285 20 289 24 293 26 581 50 585 51 589 52 593 53)
#(3 254 2 254 4 254 5 254 19 285 20 254 24 293 26 297 51 301 52 254 66)
#(3 246 2 246 4 246 5 246 19 246 24 293 26 309 52 246 66)
#(3 250 2 250 4 250 5 250 19 285 20 250 24 293 26 317 51 321 52 250 66)
#(2 238 2 4 5 19 24 66)
#(2 126 2 4 5 19 20 24 26 66)
#(3 262 2 262 4 262 5 262 19 285 20 262 24 262 26 297 51 262 66)
#(3 258 2 258 4 258 5 258 19 285 20 258 24 258 26 317 51 258 66)
#(2 234 2 4 5 19 24 26 66)
#(3 101 1 105 10 109 11 113 12 117 14 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 157 42 409 46 173 47 413 48 417 49 597 54 425 59)
#(2 58 4 5 19 66)
#(2 294 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 286 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 282 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 298 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 314 1 6 9 10 11 12 14 15 16 17 20 21 22 23 29 30 32 33 66)
#(2 290 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 278 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 302 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 270 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 306 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(2 274 1 2 3 4 5 7 8 10 11 12 14 20 21 22 23 24 26 30)
#(3 310 1 310 6 310 10 310 11 310 12 310 14 310 16 310 17 310 20 310 21 310 22 310 23 310 29 310 30 310 32 310 33 601 57 310 66)
#(2 354 66)
#(2 190 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 194 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(3 605 20 609 29)
#(2 186 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(3 370 1 370 3 370 4 370 10 370 11 370 12 370 14 370 17 370 20 370 21 370 22 370 23 370 24 370 25 370 26 370 29 370 30 370 34 613 63)
#(3 362 19 362 21 617 62)
#(3 386 1 386 3 386 4 386 10 386 11 386 12 621 13 386 14 386 17 386 20 386 21 386 22 386 23 386 24 386 25 386 26 386 29 386 30 386 34)
#(2 394 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 398 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 390 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 422 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 418 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 382 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 374 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 182 1 2 3 4 5 7 8 10 11 12 14 17 19 20 21 22 23 24 25 26 29 30 34 66)
#(2 366 19 21)
#(2 625 19)
#(3 629 9 61 20 65 29 69 32 73 33 217 59)
#(2 70 9 15 18 19)
#(3 365 5 50 19 633 39 50 66)
#(2 114 2 4 5 19 66)
#(2 218 2 4 5 19 66)
#(2 222 2 4 5 19 66)
#(3 226 2 226 4 226 5 226 19 441 24 226 66)
#(2 242 2 4 5 19 24 66)
#(3 101 1 193 6 105 10 109 11 113 12 117 14 121 16 125 17 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 637 36 149 38 153 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65 14 66)
#(2 641 8)
#(2 645 8)
#(3 101 1 509 3 649 4 517 10 109 11 113 12 117 14 521 17 525 20 129 21 133 22 137 23 529 24 533 25 537 26 541 29 141 30 545 34 549 47 553 64)
#(3 653 19 561 21)
#(2 378 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 74 2 4 5 19 20 24 26 66)
#(3 101 1 105 10 109 11 113 12 117 14 121 16 125 17 14 19 61 20 129 21 133 22 137 23 65 29 141 30 69 32 73 33 657 36 149 38 153 40 157 42 161 43 165 44 169 46 173 47 177 48 181 49 185 59 189 65)
#(2 22 19 66)
#(2 350 66)
#(2 402 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 406 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 410 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 414 1 3 4 10 11 12 14 17 20 21 22 23 24 25 26 29 30 34)
#(2 661 19)
#(2 86 2 4 5 19 20 24 26 66)
)! !

SmaCCParser subclass: #SoulParser
	instanceVariableNames: 'builder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulParser class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	
	^super new initialize! !

!SoulParser class methodsFor: 'parsing' stamp: 'sp 24/10/11 15:26'!
parseAfterRemovingSelector: aCode
	
	| selector index offset program |
	selector := Parser new parseSelector: aCode.
	index := selector size.	"JF- Adding spaces in front ensures error messages are put at the right offset"

	offset := (aCode indexOf: Character cr) - 1 max: 1.
	program := (String
		new: offset
		withAll: Character space) asText , (aCode allButFirst: index).
	^self parse: program! !

!SoulParser class methodsFor: 'generated-comments' stamp: 'AndyKellens 10/28/2011 15:22'!
parserDefinitionComment

	"%id <stterm> <quoted>;
%left "":"" ;

SOUL : PROGRAM	{#liftFirstValue:} |
		QUERY		{#liftFirstValue:};

QUERY : ""if"" TERMSEQUENCE	'termsequence'		{builder buildQuery: termsequence};

PROGRAM :	FACT 'fact' ""."" PROGRAM 'program'	{builder addTerm: fact inFront: program}|
				RULE 'rule' ""."" PROGRAM 'program'	{builder addTerm: rule inFront: program} |
				DCGRULE 'dcgrule' ""."" PROGRAM 'program' {builder addTerm: dcgrule inFront: program} |
				TABLEDRULE 'tabledrule' ""."" PROGRAM 'program' {builder addTerm: tabledrule inFront: program} |
				FACT 'fact'							{builder buildProgramSequence: fact}	|
				RULE 'rule'							{builder buildProgramSequence: rule}|
				DCGRULE 'dcgrule'					{builder buildProgramSequence: dcgrule} |
				TABLEDRULE 'tabledrule'					{builder buildProgramSequence: tabledrule};


FACT : TERM 'term'											{builder buildFact: term};

RULE : TERM 'term' ""if"" TERMSEQUENCE 'termsequence'	{builder buildRuleConclusion: term condition: termsequence};

DCGRULE : TERM 'head' <dcgoperator> TERMSEQUENCE 'body'	{builder buildDCGRule: head body: body};

TABLEDRULE :     ""tabled"" RULE 'tabledrule'                           {builder buildTabledRule: tabledrule};

TERMSEQUENCE : TERM 'term' ("","" | ""&"") TERMSEQUENCE 'termsequence'	{builder addTerm: term inFront: termsequence}|
					TERM 'term'											{builder buildTermSequence: term};

TERM : TERM 'term'  "":"" CRISPTERM 'annotation' {builder buildAnnotatedTerm: term withAnnotation: annotation}
            | CRISPTERM {#liftFirstValue:};

CRISPTERM	:	CONSTANT				{#liftFirstValue:}|
			VARIABLE				{#liftFirstValue:}|
			LIST					{#liftFirstValue:}|
			SMALLTALKTERM		{#liftFirstValue:}|
			QUOTEDCODE			{#liftFirstValue:}|
			TEMPLATEQUERY		{#liftFirstValue:}|
			CUT					{#liftFirstValue:}|
			CALLTERM				{#liftFirstValue:}|
			VARARGSCOMPOUND 	{#liftFirstValue:}|
			FUNCTIONALCOMPOUND 	{#liftFirstValue:}|
			KEYWORDEDCOMPOUND	{#liftFirstValue:}|
			MESSAGE 					{#liftFirstValue:};

KEYWORDARG :  	CONSTANT 					{#liftFirstValue:}|
					VARIABLE 					{#liftFirstValue:}|
					LIST						{#liftFirstValue:}|
					SMALLTALKTERM 			{#liftFirstValue:}|
					QUOTEDCODE 				{#liftFirstValue:}|
					TEMPLATEQUERY			{#liftFirstValue:}|
					CUT 						{#liftFirstValue:}|
					VARARGSCOMPOUND		{#liftFirstValue:}|
					FUNCTIONALCOMPOUND 	{#liftFirstValue:}|
					""("" KEYWORDEDCOMPOUND "")"" 	{#liftSecondValue:}|
					""("" KEYWORDEDMESSAGE "")"" {#liftSecondValue:}|
					""("" UNARYMESSAGE "")"" {#liftSecondValue:}|
					UNARYMESSAGE {#liftFirstValue:};

CUT : ""!!"" 	{builder buildCut};

CALLTERM : VARIABLE 'variable' ""->"" TERM 'term' {builder buildCallTerm: term variable: variable}; 

CONSTANT : <id>				{builder buildConstant: '1' value} |
			  <symbol>			{builder buildSymbol: '1' value}  ;

VARIABLE : <varid> {builder buildVariable: '1' value}|
			<posvar> {builder buildPosVariable:'1' value}|
			<negvar> {builder buildNegVariable:'1' value}|
			<delayedvar> {builder buildDelayedVariable:'1' value} |
			<underscorevar> {builder buildUnderscoreVariable:'1' value};

VARARGSCOMPOUND : FUNCTOR 'functor' ""@("" TERMSEQUENCE 'terms' "")""		{builder buildVarArgsCompound: functor withTerms: terms} ;

FUNCTIONALCOMPOUND : FUNCTOR 'functor' ""("" TERMSEQUENCE 'terms' "")""		{builder buildCompound: functor withTerms: terms} ;

KEYWORDEDCOMPOUND :  KEYWORD KEYWORDARG KEYWORDEDCOMPOUND {builder addKeyword:'1' term:'2' toKeywordedCompound:'3'} |
						      KEYWORD KEYWORDARG {builder buildKeywordedCompound:'1' firstTerm:'2'}  ;

KEYWORDEDMESSAGE : 	RECEIVER 'rec' MATCHESMESSAGEANDARGS 'mesgargs' { builder buildRegexMessageReceiver: rec message: mesgargs } |
						 	RECEIVER 'rec' KEYWORDEDMESSAGEANDARGS 'mesgargs' { builder buildKeywordedMessageReceiver: rec message: mesgargs} ;

MATCHESMESSAGEANDARGS: ""matches:"" KEYWORDARG 'arg' {builder buildKeywordedMessageFirstKey: (builder buildKeyword: #matches: ) andArgument: arg };

KEYWORDEDMESSAGEANDARGS : KEYWORD 'keyword' KEYWORDARG 'arg' KEYWORDEDMESSAGEANDARGS 'mesgargs' {builder addKeyword: keyword andArg: arg to: mesgargs} |
								     KEYWORD 'keyword' KEYWORDARG 'arg' {builder buildKeywordedMessageFirstKey: keyword andArgument: arg} ;

UNARYMESSAGE : RECEIVER 'rec' UNARYSELECTOR 'sel' {builder buildUnaryMessageReceiver: rec message: sel} ;

UNARYSELECTOR : <id> 'id' {builder buildUnaryMessageFromId: id value} | VARIABLE 'var' {builder buildUnaryMessageFromVariable: var};

RECEIVER : KEYWORDARG {#liftFirstValue:};

KEYWORD : <keyword> 	{builder buildKeyword: '1' value} |
			 <varkeyword> 	{builder buildVarKeyword: '1' value} ;

SMALLTALKTERM : <stterm> 'stterm' {builder buildSmalltalkTerm: stterm value} ;

QUOTEDCODE :  <quoted> 'quoted'  {builder buildQuotedCodeTerm: quoted value};

TEMPLATEQUERY : FUNCTIONALCOMPOUND 'type' <quoted> 'quoted' {builder buildTemplateQuery: quoted value type: type};

LIST : 	""<"" TERMSEQUENCE 'terms' "">"" {builder buildList: terms} |
		""<>""	{builder buildEmptyList} |
		""<"" TERMSEQUENCE 'terms' ""|"" TERM 'term' "">""	{builder buildPair: terms cdr: term};

FUNCTOR : 	CONSTANT 		{#liftFirstValue:}|
			VARIABLE		{#liftFirstValue:};

MESSAGE :	NONRESULTMESSAGE {#liftFirstValue:}|
			RESULTMESSAGE {#liftFirstValue:};

NONRESULTMESSAGE :	KEYWORDEDMESSAGE	{#liftFirstValue:}|
						UNARYMESSAGE		{#liftFirstValue:};

RESULTMESSAGE :	NONRESULTMESSAGE 'msg' ""="" TERM 'result' {builder buildResultMessageTerm: msg resultTerm: result};

	
	"! !

!SoulParser class methodsFor: 'generated-accessing' stamp: 'AndyKellens 10/28/2011 15:22'!
scannerClass
	^SoulScanner! !

!SoulParser class methodsFor: 'generated-starting states' stamp: 'AndyKellens 10/28/2011 15:22'!
startingStateForSOUL
	^1! !

!SoulParser methodsFor: 'comment handling' stamp: 'sp 24/10/11 15:26'!
giveAndWipeComment
	
	^scanner giveAndWipeComment! !

!SoulParser methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
initialize
	
	builder := SoulParseTreeBuilder forParser: self.
	^super initialize! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCALLTERM1: nodes
	^ builder buildCallTerm: (nodes at: 3) variable: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCONSTANT1: nodes
	^ builder buildConstant: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCONSTANT2: nodes
	^ builder buildSymbol: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForCUT1: nodes
	^ builder buildCut! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForDCGRULE1: nodes
	^ builder buildDCGRule: (nodes at: 1) body: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForFACT1: nodes
	^ builder buildFact: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForFUNCTIONALCOMPOUND1: nodes
	^ builder buildCompound: (nodes at: 1) withTerms: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForGroupXXXXXXXXXXXXXX1: nodes
	^ nodes at: 1! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForGroupXXXXXXXXXXXXXX2: nodes
	^ nodes at: 1! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORD1: nodes
	^ builder buildKeyword: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORD2: nodes
	^ builder buildVarKeyword: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORDEDCOMPOUND1: nodes
	^ builder addKeyword: (nodes at: 1) term: (nodes at: 2) toKeywordedCompound: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORDEDCOMPOUND2: nodes
	^ builder buildKeywordedCompound: (nodes at: 1) firstTerm: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORDEDMESSAGE1: nodes
	^ builder buildRegexMessageReceiver: (nodes at: 1) message: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORDEDMESSAGE2: nodes
	^ builder buildKeywordedMessageReceiver: (nodes at: 1) message: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORDEDMESSAGEANDARGS1: nodes
	^ builder addKeyword: (nodes at: 1) andArg: (nodes at: 2) to: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForKEYWORDEDMESSAGEANDARGS2: nodes
	^ builder buildKeywordedMessageFirstKey: (nodes at: 1) andArgument: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLIST1: nodes
	^ builder buildList: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLIST2: nodes
	^ builder buildEmptyList! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForLIST3: nodes
	^ builder buildPair: (nodes at: 2) cdr: (nodes at: 4)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForMATCHESMESSAGEANDARGS1: nodes
	^ builder buildKeywordedMessageFirstKey: (builder buildKeyword: #matches:) andArgument: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM1: nodes
	^ builder addTerm: (nodes at: 1) inFront: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM2: nodes
	^ builder addTerm: (nodes at: 1) inFront: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM3: nodes
	^ builder addTerm: (nodes at: 1) inFront: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM4: nodes
	^ builder addTerm: (nodes at: 1) inFront: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM5: nodes
	^ builder buildProgramSequence: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM6: nodes
	^ builder buildProgramSequence: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM7: nodes
	^ builder buildProgramSequence: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForPROGRAM8: nodes
	^ builder buildProgramSequence: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForQUERY1: nodes
	^ builder buildQuery: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForQUOTEDCODE1: nodes
	^ builder buildQuotedCodeTerm: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForRESULTMESSAGE1: nodes
	^ builder buildResultMessageTerm: (nodes at: 1) resultTerm: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForRULE1: nodes
	^ builder buildRuleConclusion: (nodes at: 1) condition: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForSMALLTALKTERM1: nodes
	^ builder buildSmalltalkTerm: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTABLEDRULE1: nodes
	^ builder buildTabledRule: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTEMPLATEQUERY1: nodes
	^ builder buildTemplateQuery: (nodes at: 2) value type: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTERM1: nodes
	^ builder buildAnnotatedTerm: (nodes at: 1) withAnnotation: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTERMSEQUENCE1: nodes
	^ builder addTerm: (nodes at: 1) inFront: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForTERMSEQUENCE2: nodes
	^ builder buildTermSequence: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForUNARYMESSAGE1: nodes
	^ builder buildUnaryMessageReceiver: (nodes at: 1) message: (nodes at: 2)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForUNARYSELECTOR1: nodes
	^ builder buildUnaryMessageFromId: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForUNARYSELECTOR2: nodes
	^ builder buildUnaryMessageFromVariable: (nodes at: 1)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVARARGSCOMPOUND1: nodes
	^ builder buildVarArgsCompound: (nodes at: 1) withTerms: (nodes at: 3)! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVARIABLE1: nodes
	^ builder buildVariable: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVARIABLE2: nodes
	^ builder buildPosVariable: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVARIABLE3: nodes
	^ builder buildNegVariable: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVARIABLE4: nodes
	^ builder buildDelayedVariable: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-reduction actions' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceActionForVARIABLE5: nodes
	^ builder buildUnderscoreVariable: (nodes at: 1) value! !

!SoulParser methodsFor: 'generated-tables' stamp: 'AndyKellens 10/28/2011 15:22'!
reduceTable
	^#(
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(32 1 #liftFirstValue:)
#(33 1 #reduceActionForCONSTANT1:)
#(33 1 #reduceActionForCONSTANT2:)
#(34 3 #reduceActionForTERM1:)
#(34 1 #liftFirstValue:)
#(35 3 #reduceActionForLIST1:)
#(35 1 #reduceActionForLIST2:)
#(35 5 #reduceActionForLIST3:)
#(36 1 #reduceActionForSMALLTALKTERM1:)
#(37 1 #reduceActionForQUOTEDCODE1:)
#(38 2 #reduceActionForTEMPLATEQUERY1:)
#(39 1 #reduceActionForCUT1:)
#(40 3 #reduceActionForCALLTERM1:)
#(41 4 #reduceActionForVARARGSCOMPOUND1:)
#(42 4 #reduceActionForFUNCTIONALCOMPOUND1:)
#(43 3 #reduceActionForKEYWORDEDCOMPOUND1:)
#(43 2 #reduceActionForKEYWORDEDCOMPOUND2:)
#(44 1 #liftFirstValue:)
#(44 1 #liftFirstValue:)
#(45 2 #reduceActionForKEYWORDEDMESSAGE1:)
#(45 2 #reduceActionForKEYWORDEDMESSAGE2:)
#(46 2 #reduceActionForUNARYMESSAGE1:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 1 #liftFirstValue:)
#(47 3 #liftSecondValue:)
#(47 3 #liftSecondValue:)
#(47 3 #liftSecondValue:)
#(47 1 #liftFirstValue:)
#(48 2 #reduceActionForTABLEDRULE1:)
#(49 3 #reduceActionForDCGRULE1:)
#(50 3 #reduceActionForRULE1:)
#(51 1 #reduceActionForFACT1:)
#(52 3 #reduceActionForTERMSEQUENCE1:)
#(52 1 #reduceActionForTERMSEQUENCE2:)
#(53 1 #reduceFor:)
#(54 1 #liftFirstValue:)
#(54 1 #liftFirstValue:)
#(55 1 #liftFirstValue:)
#(55 1 #liftFirstValue:)
#(56 1 #reduceActionForKEYWORD1:)
#(56 1 #reduceActionForKEYWORD2:)
#(57 1 #liftFirstValue:)
#(58 2 #reduceActionForMATCHESMESSAGEANDARGS1:)
#(59 3 #reduceActionForKEYWORDEDMESSAGEANDARGS1:)
#(59 2 #reduceActionForKEYWORDEDMESSAGEANDARGS2:)
#(60 1 #reduceActionForUNARYSELECTOR1:)
#(60 1 #reduceActionForUNARYSELECTOR2:)
#(61 2 #reduceActionForQUERY1:)
#(62 3 #reduceActionForPROGRAM1:)
#(62 3 #reduceActionForPROGRAM2:)
#(62 3 #reduceActionForPROGRAM3:)
#(62 3 #reduceActionForPROGRAM4:)
#(62 1 #reduceActionForPROGRAM5:)
#(62 1 #reduceActionForPROGRAM6:)
#(62 1 #reduceActionForPROGRAM7:)
#(62 1 #reduceActionForPROGRAM8:)
#(63 1 #liftFirstValue:)
#(63 1 #liftFirstValue:)
#(64 3 #reduceActionForRESULTMESSAGE1:)
#(67 1 #reduceActionForGroupXXXXXXXXXXXXXX1:)
#(67 1 #reduceActionForGroupXXXXXXXXXXXXXX2:)
#(68 1 #reduceActionForVARIABLE1:)
#(68 1 #reduceActionForVARIABLE2:)
#(68 1 #reduceActionForVARIABLE3:)
#(68 1 #reduceActionForVARIABLE4:)
#(68 1 #reduceActionForVARIABLE5:)
)! !

!SoulParser methodsFor: 'generated-tables' stamp: 'AndyKellens 10/28/2011 15:22'!
transitionTable
	^#(
#(3 9 2 13 3 17 7 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 85 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 141 48 145 49 149 50 153 51 157 54 161 55 165 56 169 57 173 61 177 62 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 193 33 197 35 201 36 205 37 209 38 213 39 217 41 221 42 225 43 229 45 233 46 137 47 161 55 165 56 169 57 237 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 241 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 245 50 161 55 165 56 169 57 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 253 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(2 74 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 94 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 257 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(2 54 1 2 4 5 6 7 8 9 11 12 15 17 18 19 21 22 23 24 25 26 29 65)
#(2 234 2 10 13 14 18 20 21 23 24 25 26 27 28)
#(2 58 1 2 4 5 6 7 8 9 11 12 15 17 18 19 21 22 23 24 25 26 29 65)
#(2 322 1 2 4 5 6 7 8 9 11 12 15 16 17 18 19 21 22 23 24 25 26 29 65)
#(2 238 2 10 13 14 18 20 21 23 24 25 26 27 28)
#(2 334 1 2 4 5 6 7 8 9 11 12 15 16 17 18 19 21 22 23 24 25 26 29 65)
#(2 326 1 2 4 5 6 7 8 9 11 12 15 16 17 18 19 21 22 23 24 25 26 29 65)
#(2 330 1 2 4 5 6 7 8 9 11 12 15 16 17 18 19 21 22 23 24 25 26 29 65)
#(2 338 1 2 4 5 6 7 8 9 11 12 15 16 17 18 19 21 22 23 24 25 26 29 65)
#(2 82 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 86 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 66 1 4 5 6 7 9 11 12 29 65)
#(3 6 1 226 2 6 4 6 5 6 6 6 7 6 9 6 11 6 12 226 15 138 17 138 18 138 19 138 21 138 22 138 23 138 24 138 25 138 26 6 29 6 65)
#(3 261 6 265 7 202 9 269 29 202 65)
#(3 14 1 14 4 14 5 14 6 14 7 14 9 14 11 14 12 146 17 146 18 146 19 146 21 146 22 146 23 146 24 146 25 146 26 14 29 14 65)
#(3 18 1 18 4 18 5 18 6 18 7 18 9 18 11 18 12 150 17 150 18 150 19 150 21 150 22 150 23 150 24 150 25 150 26 18 29 18 65)
#(3 22 1 22 4 22 5 22 6 22 7 22 9 22 11 22 12 154 17 154 18 154 19 154 21 154 22 154 23 154 24 154 25 154 26 22 29 22 65)
#(3 26 1 26 4 26 5 26 6 26 7 26 9 26 11 26 12 158 17 158 18 158 19 158 21 158 22 158 23 158 24 158 25 158 26 26 29 26 65)
#(3 30 1 30 4 30 5 30 6 30 7 30 9 30 11 30 12 162 17 162 18 162 19 162 21 162 22 162 23 162 24 162 25 162 26 30 29 30 65)
#(2 34 1 4 5 6 7 9 11 12 29 65)
#(3 38 1 38 4 38 5 38 6 38 7 38 9 38 11 38 12 166 17 166 18 166 19 166 21 166 22 166 23 166 24 166 25 166 26 38 29 38 65)
#(3 42 1 42 4 42 5 42 6 42 7 42 9 42 11 42 12 170 17 170 18 170 19 170 21 170 22 170 23 170 24 170 25 170 26 273 28 42 29 42 65)
#(2 46 1 4 5 6 7 9 11 12 29 65)
#(2 50 1 4 5 6 7 9 11 12 29 65)
#(2 302 1 4 5 6 7 8 9 11 12 29 65)
#(3 306 1 306 4 306 5 306 6 306 7 306 8 306 9 306 11 306 12 186 17 186 18 186 19 186 21 186 22 186 23 186 24 186 25 186 26 306 29 306 65)
#(2 242 17 18 19 21 22 23 24 25 26)
#(3 277 9 298 65)
#(3 281 9 294 65)
#(3 285 9 290 65)
#(3 289 9 286 65)
#(2 0 65)
#(3 293 2 297 15)
#(3 9 2 21 10 25 13 29 14 33 18 41 20 45 21 53 23 57 24 61 25 65 26 69 27 73 28 193 33 197 35 201 36 205 37 209 38 213 39 217 41 221 42 301 46 305 47 161 55 309 57 237 68)
#(3 313 17 317 18 37 19 45 21 49 22 53 23 57 24 61 25 65 26 321 56 325 58 329 59 333 60 337 68)
#(2 222 65)
#(2 218 65)
#(3 118 1 118 4 118 5 118 6 118 7 341 8 118 9 118 11 118 12 118 29 118 65)
#(2 122 1 4 5 6 7 9 11 12 29 65)
#(3 10 1 230 2 10 4 10 5 10 6 10 7 10 9 10 11 10 12 230 15 345 16 142 17 142 18 142 19 142 21 142 22 142 23 142 24 142 25 142 26 10 29 10 65)
#(3 138 1 226 2 138 4 138 5 138 6 138 7 138 8 138 9 138 11 138 12 226 15 138 17 138 18 138 19 138 21 138 22 138 23 138 24 138 25 138 26 138 29 138 65)
#(2 146 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 150 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 154 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 158 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 162 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 166 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(3 170 1 170 4 170 5 170 6 170 7 170 8 170 9 170 11 170 12 170 17 170 18 170 19 170 21 170 22 170 23 170 24 170 25 170 26 273 28 170 29 170 65)
#(2 349 5)
#(2 353 5)
#(3 357 5 186 17 186 18 186 19 186 21 186 22 186 23 186 24 186 25 186 26)
#(3 142 1 230 2 142 4 142 5 142 6 142 7 142 8 142 9 142 11 142 12 230 15 142 17 142 18 142 19 142 21 142 22 142 23 142 24 142 25 142 26 142 29 142 65)
#(3 261 6 265 7)
#(2 190 9 65)
#(3 361 1 365 4 210 5 261 6 210 9 210 11 210 12 210 65 369 67)
#(2 266 65)
#(3 373 11 377 12)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 381 32 81 33 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 161 55 165 56 169 57 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 385 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 389 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(2 90 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(3 9 2 13 3 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 85 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 141 48 145 49 149 50 153 51 161 55 165 56 169 57 393 62 181 63 185 64 189 68)
#(3 9 2 13 3 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 85 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 141 48 145 49 149 50 153 51 161 55 165 56 169 57 397 62 181 63 185 64 189 68)
#(3 9 2 13 3 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 85 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 141 48 145 49 149 50 153 51 161 55 165 56 169 57 401 62 181 63 185 64 189 68)
#(3 9 2 13 3 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 85 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 141 48 145 49 149 50 153 51 161 55 165 56 169 57 405 62 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 409 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 413 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(2 186 1 4 5 6 7 8 9 11 12 18 19 21 22 23 24 25 26 29 65)
#(3 114 1 114 4 114 5 114 6 114 7 114 9 114 11 114 12 242 18 37 19 242 21 49 22 242 23 242 24 242 25 242 26 114 29 417 43 165 56 114 65)
#(3 317 18 45 21 53 23 57 24 61 25 65 26 333 60 337 68)
#(3 9 2 21 10 25 13 29 14 33 18 41 20 45 21 53 23 57 24 61 25 65 26 69 27 73 28 193 33 197 35 201 36 205 37 209 38 213 39 217 41 221 42 301 46 421 47 161 55 309 57 237 68)
#(2 258 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(3 9 2 21 10 25 13 29 14 33 18 41 20 45 21 53 23 57 24 61 25 65 26 69 27 73 28 193 33 197 35 201 36 205 37 209 38 213 39 217 41 221 42 301 46 425 47 161 55 309 57 237 68)
#(2 126 1 4 5 6 7 8 9 11 12 29 65)
#(2 130 1 4 5 6 7 8 9 11 12 29 65)
#(2 134 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 262 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 429 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 161 55 165 56 169 57 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 433 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 161 55 165 56 169 57 181 63 185 64 189 68)
#(2 174 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 178 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 182 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 318 2 10 13 14 18 19 20 21 22 23 24 25 26 27 28)
#(2 314 2 10 13 14 18 19 20 21 22 23 24 25 26 27 28)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 249 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 437 52 161 55 165 56 169 57 181 63 185 64 189 68)
#(3 9 2 21 10 25 13 29 14 33 18 37 19 41 20 45 21 49 22 53 23 57 24 61 25 65 26 69 27 73 28 77 32 81 33 441 34 89 35 93 36 97 37 101 38 105 39 109 40 113 41 117 42 121 43 125 44 129 45 133 46 137 47 161 55 165 56 169 57 181 63 185 64 189 68)
#(2 70 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 62 1 4 5 6 7 9 11 12 29 65)
#(2 198 9 65)
#(2 194 9 65)
#(2 282 65)
#(2 278 65)
#(2 274 65)
#(2 270 65)
#(2 445 5)
#(2 449 5)
#(2 110 1 4 5 6 7 9 11 12 29 65)
#(3 246 1 246 4 246 5 246 6 246 7 246 8 246 9 246 11 246 12 242 18 242 21 242 23 242 24 242 25 242 26 246 29 246 65)
#(3 254 1 254 4 254 5 254 6 254 7 254 8 254 9 254 11 254 12 242 18 37 19 242 21 49 22 242 23 242 24 242 25 242 26 254 29 321 56 453 59 254 65)
#(3 310 1 310 4 310 5 261 6 310 7 310 9 310 11 310 12 310 29 310 65)
#(3 98 1 98 4 98 5 261 6 98 7 98 9 98 11 98 12 98 29 98 65)
#(2 206 5 9 11 12 65)
#(3 261 6 457 12)
#(2 106 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 28 29 65)
#(2 102 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
#(2 250 1 4 5 6 7 8 9 11 12 29 65)
#(2 78 1 4 5 6 7 8 9 11 12 17 18 19 21 22 23 24 25 26 29 65)
)! !

RBProgramNodeVisitor subclass: #RBSoulVariableExtractorVisitor
	instanceVariableNames: 'dict'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!RBSoulVariableExtractorVisitor class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	
	^super new initialize! !

!RBSoulVariableExtractorVisitor methodsFor: 'initialize' stamp: 'sp 24/10/11 15:26'!
initialize
	
	dict := Dictionary new! !

!RBSoulVariableExtractorVisitor methodsFor: 'access' stamp: 'sp 24/10/11 15:26'!
variables
	
	^dict values asOrderedCollection! !

!RBSoulVariableExtractorVisitor methodsFor: 'visitor - double dispatching' stamp: 'AndyKellens 11/2/2011 09:53'!
visitDelayedSoulVariable: aSoulVariable
	
	dict
		at: aSoulVariable name
		put: (SoulFactory current makeDelayedVariable name: aSoulVariable name)! !

!RBSoulVariableExtractorVisitor methodsFor: 'visitor - double dispatching' stamp: 'AndyKellens 11/2/2011 09:53'!
visitSoulVariable: aSoulVariable
	
	dict
		at: aSoulVariable name
		put: (SoulFactory current makeVariable name: aSoulVariable name)! !

RBProgramNodeVisitor subclass: #SmalltalkParseTreeConverter
	instanceVariableNames: 'methodClassTerm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptArrayNode: anArrayNode
	"This not right yet... this code merely avoids an error"

	
	^self
		makeCompoundTerm: 'braces'
		withTerms: (self visitNodes: anArrayNode messages)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptAssignmentNode: anAssignmentNode
	
	^self
		makeCompoundTerm: 'assign'
		withTerm: (self visitNode: anAssignmentNode variable)
		andTerm: (self visitNode: anAssignmentNode value)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptBlockNode: aBlockNode
	
	| temp |
	temp := self visitNode: aBlockNode body.
	^self
		makeCompoundTerm: 'block'
		withTerm:
			(self
				makeCompoundTerm: 'arguments'
				withTerm: (self visitNodes: aBlockNode arguments))
		andTerm: temp first
		andTerm: (temp at: 2)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'AndyKellens 11/2/2011 09:53'!
acceptCascadeNode: aCascadeNode
	
	| sendInfo arguments receiver |
	receiver := self visitNode: aCascadeNode receiver.
	arguments := aCascadeNode messages
		collect:
			[:msg | 
			sendInfo := Array
				with: receiver
				with: (Constant name: msg selector)
				with: (self visitNodes: msg arguments).
			self makeCompoundTerm: 'cascadeSend' withTerms: sendInfo].
	^self
		makeCompoundTerm: 'cascade'
		withTerm: receiver
		andTerm: (SoulFactory current makeList termSequence: arguments)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptLiteralArrayNode: aRBLiteralArrayNode
	"This not right yet... this code merely avoids an error"

	
	^self
		makeCompoundTerm: 'array'
		withTerms:
			(OrderedCollection with: (self visitNodes: aRBLiteralArrayNode contents))! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'AndyKellens 11/2/2011 09:53'!
acceptLiteralNode: aLiteralNode
	
	^self
		makeCompoundTerm: 'literal'
		withTerm: (SoulFactory current makeUppedObject up: aLiteralNode value)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptMessageNode: aMessageNode
	
	^self
		makeCompoundTerm: 'send'
		withTerm: (self visitNode: aMessageNode receiver)
		andTerm: (Constant name: aMessageNode selector asSymbol)
		andTerm: (self visitNodes: aMessageNode arguments)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptMethodNode: aMethodNode
	
	| temp |
	temp := self visitNode: aMethodNode body.
	^self
		makeProgramSequence:
			(self
				makeCompoundTerm: 'method'
				withTerm: self methodClassTerm
				andTerm: (Constant name: aMethodNode selector asSymbol)
				andTerm:
					(self
						makeCompoundTerm: 'arguments'
						withTerm: (self visitNodes: aMethodNode arguments))
				andTerm: temp first
				andTerm: (temp at: 2))! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptOptimizedNode: anOptimizedNode
	
	self error: 'Optimized nodes can not be handled by the Converter for now'! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptReturnNode: aReturnNode
	
	^self
		makeCompoundTerm: 'return'
		withTerm: (self visitNode: aReturnNode value)! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptSequenceNode: aSequenceNode
	
	^OrderedCollection
		with:
			(self
				makeCompoundTerm: 'temporaries'
				withTerm: (self visitNodes: aSequenceNode temporaries))
		with:
			(self
				makeCompoundTerm: 'statements'
				withTerm: (self visitNodes: aSequenceNode statements))! !

!SmalltalkParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptVariableNode: aVariableNode
	
	^self
		makeCompoundTerm: 'variable'
		withTerm: (Constant name: aVariableNode name asSymbol)! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
defaultMethodClassTerm
	
	^Variable name: 'unknownMethodClass'! !

!SmalltalkParseTreeConverter methodsFor: 'public access' stamp: 'AndyKellens 11/2/2011 09:53'!
doClass: aClass selector: aSymbol
	
	| source |
	methodClassTerm := SoulFactory current makeUppedObject up: aClass.
	source := aClass compilerClass = Object compilerClass
		ifTrue: [(aClass sourceCodeAt: aSymbol) string]
		ifFalse: [aSymbol asString].
	^self doString: source inClass: aClass! !

!SmalltalkParseTreeConverter methodsFor: 'public access' stamp: 'sp 24/10/11 15:26'!
doString: aString
	
	^self startForNode: (RBParser parseMethod: aString)! !

!SmalltalkParseTreeConverter methodsFor: 'public access' stamp: 'AndyKellens 11/2/2011 09:53'!
doString: aString inClass: aClass
	
	methodClassTerm := SoulFactory current makeUppedObject up: aClass.
	^self doString: aString! !

!SmalltalkParseTreeConverter methodsFor: 'initialize-release' stamp: 'sp 24/10/11 15:26'!
initialize
	
	^self! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeCompoundTerm: aString withTerm: aTerm
	
	^SoulFactory current makeCompound
		functor: (Constant name: aString asSymbol)
		termSequence: (TermSequence terms: (OrderedCollection with: aTerm))! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeCompoundTerm: aString withTerm: aTerm andTerm: aSecondTerm
	
	^SoulFactory current makeCompound
		functor: (Constant name: aString asSymbol)
		termSequence:
			(TermSequence terms: (OrderedCollection with: aTerm with: aSecondTerm))! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeCompoundTerm: aString withTerm: aTerm andTerm: aSecondTerm andTerm: aThirdTerm
	
	^SoulFactory current makeCompound
		functor: (Constant name: aString asSymbol)
		termSequence:
			(TermSequence
				terms:
					(OrderedCollection with: aTerm with: aSecondTerm with: aThirdTerm))! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeCompoundTerm: aString withTerm: aTerm andTerm: aSecondTerm andTerm: aThirdTerm andTerm: aFourthTerm
	
	^SoulFactory current makeCompound
		functor: (Constant name: aString asSymbol)
		termSequence:
			(TermSequence
				terms:
					(OrderedCollection with: aTerm with: aSecondTerm with: aThirdTerm with:
							aFourthTerm))! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeCompoundTerm: aString withTerm: aTerm andTerm: aSecondTerm andTerm: aThirdTerm andTerm: aFourthTerm andTerm: aFifthTerm
	
	^SoulFactory current makeCompound
		functor: (Constant name: aString asSymbol)
		termSequence:
			(TermSequence
				terms:
					((OrderedCollection new: 5)
						add: aTerm;
						add: aSecondTerm;
						add: aThirdTerm;
						add: aFourthTerm;
						add: aFifthTerm;
						yourself))! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
makeCompoundTerm: aString withTerms: aCollectionOfTerms
	
	^SoulFactory current makeCompound
		functor: (Constant name: aString asSymbol)
		termSequence: (TermSequence terms: aCollectionOfTerms)! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
makeFact: aClause
	
	^Fact
		condition: TermSequence new
		conclusion: aClause! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
makeProgramSequence: aClause
	
	^ProgramSequence
		clauses: (OrderedCollection with: (self makeFact: aClause))! !

!SmalltalkParseTreeConverter methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
methodClassTerm
	
	^methodClassTerm isNil
		ifTrue: [self defaultMethodClassTerm]
		ifFalse: [methodClassTerm]! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
startForNode: aNode
	
	^self visitNode: aNode! !

!SmalltalkParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
stringForClass: aClass
	"return the string that will be put as the class argument in the internal 
	parsetree "

	
	^aClass printString! !

!SmalltalkParseTreeConverter methodsFor: 'visiting' stamp: 'sp 24/10/11 15:26'!
visitNode: aNode
	
	^aNode acceptVisitor: self! !

!SmalltalkParseTreeConverter methodsFor: 'visiting' stamp: 'AndyKellens 11/2/2011 09:53'!
visitNodes: aNodeCollection
	
	^SoulFactory current makeList
		termSequence:
			(TermSequence
				terms: (aNodeCollection collect: [:node | self visitNode: node]))! !

SmalltalkParseTreeConverter subclass: #SmalltalkPatternParseTreeConverter
	instanceVariableNames: 'environment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SmalltalkPatternParseTreeConverter class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
inEnvironment: anEnvironment
	
	^self new environment: anEnvironment! !

!SmalltalkPatternParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptMessageNode: aMessageNode
	
	^aMessageNode isPatternNode
		ifTrue: [self processPatternMessageNode: aMessageNode]
		ifFalse: [super acceptMessageNode: aMessageNode]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptMethodNode: aMethodNode
	
	^aMethodNode isPatternNode
		ifTrue: [self processPatternMethodNode: aMethodNode]
		ifFalse: [super acceptMethodNode: aMethodNode]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'visitor-double dispatching' stamp: 'sp 24/10/11 15:26'!
acceptVariableNode: aVariableNode
	
	^aVariableNode isPatternNode
		ifTrue: [self processPatternVariable: aVariableNode]
		ifFalse: [super acceptVariableNode: aVariableNode]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
environment
	
	^environment! !

!SmalltalkPatternParseTreeConverter methodsFor: 'accessing' stamp: 'sp 24/10/11 15:26'!
environment: anObject
	
	environment := anObject! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processArgument: anArgument
	
	^anArgument isPatternNode
		ifTrue: [self processPatternVariable: anArgument]
		ifFalse: [self visitNode: anArgument]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processArguments: aMessageNode
	
	^aMessageNode arguments size > 1
		ifTrue: [self processMultiArguments: aMessageNode arguments]
		ifFalse: [self processArgument: (aMessageNode arguments at: 1)]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processKeyword: anRBKeyword
	
	^anRBKeyword isPatternVariable
		ifTrue: [self processSelectorVariable: anRBKeyword]
		ifFalse: [anRBKeyword value]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
processMultiArgumentSelector: aCollection
	
	^SoulFactory current makeList
		termSequence:
			(TermSequence
				terms: (aCollection collect: [:each | self processKeyword: each]))! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
processMultiArguments: aCollection
	
	^SoulFactory current makeList
		termSequence:
			(TermSequence
				terms: (aCollection collect: [:each | self processArgument: each]))! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processPatternMessageNode: aMessageNode
	
	^self
		makeCompoundTerm: 'send'
		withTerm: (self visitNode: aMessageNode receiver)
		andTerm: (self processSelector: aMessageNode)
		andTerm: (self processArguments: aMessageNode)! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processPatternMethodNode: aMethodNode
	
	self halt! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processPatternVariable: aVariableNode
	
	| name |
	name := self skipNonAlphabeticCharacters: aVariableNode name.
	^aVariableNode isStatement
		ifTrue:
			[self
				makeCompoundTerm: 'listOfStatements'
				withTerm: (Variable name: name)]
		ifFalse: [Variable name: name]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processSelector: aMessageNode
	
	^aMessageNode selectorParts size > 1
		ifTrue: [self processMultiArgumentSelector: aMessageNode selectorParts]
		ifFalse:
			[self processSingleArgumentSelector: (aMessageNode selectorParts at: 1)]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
processSelectorVariable: anRBKeywordToken
	
	^Variable name: (self skipNonAlphabeticCharacters: anRBKeywordToken value)! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'AndyKellens 11/2/2011 09:53'!
processSingleArgumentSelector: anRBKeyword
	
	^anRBKeyword isPatternVariable
		ifTrue:
			[| var |
			var := RBPatternVariableNode identifierToken: anRBKeyword.
			var isList
				ifTrue: [self processSelectorVariable: anRBKeyword]
				ifFalse:
					[SoulFactory current makeList
						termSequence:
							(TermSequence
								terms:
									(OrderedCollection with: (self processSelectorVariable: anRBKeyword)))]]
		ifFalse: [anRBKeyword value]! !

!SmalltalkPatternParseTreeConverter methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
skipNonAlphabeticCharacters: aString
	
	^aString select: [:el | el isAlphaNumeric]! !

Array variableSubclass: #ScannerTable
	instanceVariableNames: 'value0 defaultValue letterValue digitValue separatorValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!ScannerTable commentStamp: '<historical>' prior: 0!
I hold the table that the Scanner or similar application uses to look up characters by character value.  If the character value is out of range, I answer one of several default values, depending on how the character classifies itself.

Instance Variables:
	value0	<Object>  "an explicit entry for index=0"
	defaultValue	<Object>  "the value for undefined characters"
	letterValue	<Object>  "the default value for letters"
	digitValue	<Object>  "the default value for digits"
	separatorValue	<Object>  "the default value for separators (whitespace)"

Note that we have to override at:put: to make the defaulting mechanism work properly, because at: and at:put: both send subscriptBoundsError:, which loses both the information as to which message was sent and (in the case of at:put:) what value was being stored.  This is a design bug in the implementation of at: and at:put:.!

!ScannerTable methodsFor: 'accessing-defaults' stamp: 'sp 24/10/11 15:26'!
atAllDigitsPut: value
	" Set the value for all Characters
	that identify themselves as digits
	and that are outside the range of
	explicitly stored indices. "

	
	digitValue := value.
	0
		to: self size
		do:
			[:index | (Character value: index) isDigit ifTrue: [self at: index put: value]]! !

!ScannerTable methodsFor: 'accessing-defaults' stamp: 'sp 24/10/11 15:26'!
atAllLettersPut: value
	" Set the value for all Characters
	that identify themselves as letters
	and that are outside the range of
	explicitly stored indices. "

	
	letterValue := value.
	0
		to: self size
		do:
			[:index | (Character value: index) isLetter ifTrue: [self at: index put: value]]! !

!ScannerTable methodsFor: 'accessing-defaults' stamp: 'sp 24/10/11 15:26'!
atAllPut: value
	
	value0 := defaultValue := letterValue := digitValue := separatorValue := value.
	^super atAllPut: value! !

!ScannerTable methodsFor: 'accessing-defaults' stamp: 'sp 24/10/11 15:26'!
atAllSeparatorsPut: value
	" Set the value for all Characters
	that identify themselves as separators
	and that are outside the range of
	explicitly stored indices. "

	
	separatorValue := value.
	0
		to: self size
		do:
			[:index | (Character value: index) isSeparator ifTrue: [self at: index put: value]]! !

!ScannerTable methodsFor: 'accessing-defaults' stamp: 'sp 24/10/11 15:26'!
atDefaultPut: value
	" Set the value for all Characters
	that are not letters and that are
	outside the range of explicitly stored
	indices. "

	
	defaultValue := value! !

!ScannerTable methodsFor: 'error handling' stamp: 'sp 24/10/11 15:26'!
subscriptBoundsErrorFor: selector index: index
	"This method is invoked when a subscript is an integer but is out of range.
	 Answer an appropriate default value."

	
	| char |
	index = 0 ifTrue: [^value0].
	(index
		between: self size + 1
		and: Character maxValue)
		ifFalse: [^super subscriptBoundsErrorFor: selector index: index].
	char := Character value: index.
	char isLetter ifTrue: [^letterValue].
	char isSeparator ifTrue: [^separatorValue].
	char isDigit ifTrue: [^digitValue].
	^defaultValue! !

!ScannerTable methodsFor: 'error handling' stamp: 'sp 24/10/11 15:26'!
subscriptBoundsErrorFor: selector index: index value: aValue
	"This method is invoked when a subscript is an integer
	 but is out of range during an at:put:.  If the index is zero
	 assign the zero value, otherwise raise an error."

	
	index = 0 ifTrue: [^value0 := aValue].
	^super subscriptBoundsErrorFor: selector index: index value: aValue! !

RBPatternScanner subclass: #SoulRBPatternScanner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulRBPatternScanner class methodsFor: 'class initialization' stamp: 'sp 24/10/11 15:26'!
initializeClassificationTable
	"self initialize"

	
	super initializeClassificationTable	"PatternVariableCharacter := $?"! !

RBVariableNode subclass: #RBSoulVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

RBSoulVariableNode subclass: #RBDelayedSoulVariableNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!RBDelayedSoulVariableNode methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
acceptVisitor: aVisitor
	
	^aVisitor visitDelayedSoulVariable: self! !

!RBSoulVariableNode class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
identifierToken: anIdentifierToken
	"Explicitly override this method for compatibility in Squeak. The Squeak version of RBVariableNode>>identifierToken explicitly makes either an instance of RBPatternVariableNode or an instance of RBVariableNode."

	
	^self new identifierToken: anIdentifierToken! !

!RBSoulVariableNode methodsFor: 'visitor' stamp: 'sp 24/10/11 15:26'!
acceptVisitor: aVisitor
	
	^aVisitor visitSoulVariable: self! !

Object subclass: #SoulMethodCompiler
	instanceVariableNames: 'endOfLastToken'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulMethodCompiler class methodsFor: 'signal constants' stamp: 'sp 24/10/11 15:26'!
errorSignal
	
	^SoulError! !

!SoulMethodCompiler class methodsFor: 'defaults' stamp: 'sp 24/10/11 15:26'!
preferredParserClass
	
	^self! !

!SoulMethodCompiler methodsFor: 'compilation' stamp: 'AndyKellens 10/28/2011 13:08'!
compile: aCode in: aClass classified: category notifying: aRequestor ifFail: aFailBlock
	
	| selector result |
	selector := Parser new parseSelector: aCode.
	result := Compiler new
		compile:
			selector , ' ^ self class parseInstanceStorageMethod: '
				, selector printString
		in: aClass
		notifying: nil
		ifFail: [^nil].
	"Change the source pointer of the method node"
	result sourceText: aCode.
	aClass doingInternalStorage
		ifFalse:
			["TODO: Is this the best place to do this? At this moment the actual code of the storage method has *not* yet been changed!!"

			aClass storageChangedExternally: selector newCode: aCode].
	^result! !

!SoulMethodCompiler methodsFor: 'compilation' stamp: 'sp 24/10/11 15:26'!
compile: textOrStream in: aClass notifying: aRequestor ifFail: failBlock
	
	^self compile: textOrStream in: aClass classified: nil notifying:
			aRequestor ifFail: failBlock! !

!SoulMethodCompiler methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
endOfLastToken
	"The bad VW implementation of Text>>makeSelectorBoldIn: gives 
	trouble. It needs to emphasize the selector of a method in 
	bold. Therefore it first parses the selector (which, as a side 
	effect means that the parser holds on to the position just after 
	the selector). Then, this position is asked for with the general 
	#endOfLastToken message (hence assuming lots of 
	implementation details of the parser)."

	"We mimic this behavior by setting an instance variable with 
	the position of the end of the selector. This is then returned 
	when the #endOfLastToken message is called."

	
	| end |
	end := endOfLastToken.
	end isNil
		ifTrue:
			[self error:
					'SOULMethodParser is in trouble (endOfLastToken is being used where it wasn''t supposed to!!)']
		ifFalse:
			[endOfLastToken := nil.
			^end]! !

!SoulMethodCompiler methodsFor: 'compilation' stamp: 'sp 24/10/11 15:26'!
environment: dontCare
	
	^nil! !

!SoulMethodCompiler methodsFor: 'compilation' stamp: 'sp 24/10/11 15:26'!
format: textOrStream in: aClass notifying: aRequestor decorated: aBoolean
	
	^textOrStream! !

!SoulMethodCompiler methodsFor: 'public accessing' stamp: 'sp 24/10/11 15:26'!
parse: aStream class: class environment: environment noPattern: noPattern context: context notifying: handler builder: aProgramNodeBuilder saveComments: mapFlag ifFail: failBlock
	
	^Parser new parse: aStream class: class environment: environment
		noPattern: noPattern context: context notifying: handler builder:
			aProgramNodeBuilder saveComments: mapFlag ifFail: failBlock! !

!SoulMethodCompiler methodsFor: 'compilation' stamp: 'sp 24/10/11 15:26'!
parse: aStream class: class noPattern: noPattern context: context notifying: handler builder: builder saveComments: saveComments ifFail: failblock
	"this is invoked during the compilation of pieces of QSoul code, after the QSoul code was converted to a Smalltalk representation. Hence the regular Smalltalk parser should be used to handle this."

	
	^Parser new parse: aStream class: class noPattern: noPattern context:
			context notifying: handler builder: builder saveComments: saveComments
		ifFail: failblock! !

!SoulMethodCompiler methodsFor: 'compilation' stamp: 'sp 24/10/11 15:26'!
parse: aStream class: class noPattern: noPattern context: context notifying: handler ifFail: failblock
	
	^Parser new
		parse: aStream
		class: class
		noPattern: noPattern
		context: context
		notifying: handler
		builder: ProgramNodeBuilder new
		saveComments: true
		ifFail: failblock! !

!SoulMethodCompiler methodsFor: 'public accessing' stamp: 'sp 24/10/11 15:26'!
parse: textOrStream in: aClass environment: env allowReceiver: allowSelf noPattern: noPattern notifying: req
	"Compile the incoming text and answer with the resulting parse tree.
	This is the only public message that saves the source map
	and the comments.  If the requestor is nil,
	warning messages (as opposed to outright errors)
	are silently discarded."

	
	^Compiler new parse: textOrStream in: aClass environment: env
		allowReceiver: allowSelf noPattern: noPattern notifying: req! !

!SoulMethodCompiler methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
parseSelector: aString
	"The bad VW implementation of Text>>makeSelectorBoldIn: gives 
	trouble. It needs to emphasize the selector of a method in 
	bold. Therefore it first parses the selector (which, as a side 
	effect means that the parser holds on to the position just after 
	the selector). Then, this position is asked for with the general 
	#endOfLastToken message (hence assuming lots of 
	implementation details of the parser)."

	"Here we mimic this behavior by setting an instance variable with 
	the position of the end of the selector. This is then returned 
	when the #endOfLastToken message is called."

	
	| parser sel |
	parser := Parser new.
	sel := parser parseSelector: aString.
	endOfLastToken := parser endOfLastToken.
	^sel! !

!SoulMethodCompiler methodsFor: 'public accessing' stamp: 'sp 24/10/11 15:26'!
parseSelectorInterval: aString
	"inspired from Parser"

	
	| selector |
	selector := self parseSelector: aString.
	^selector isNil
		ifTrue: [1 to: 0]
		ifFalse: [1 to: selector size]! !

!SoulMethodCompiler methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
predicateKeyAsSelector: aKey
	
	^(((aKey copyReplaceAll: ':' with: 'X') copyReplaceAll: '/' with: 'M')
		copyReplaceAll: '=' with: 'E') asSymbol! !

Object subclass: #SoulParseTreeBuilder
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOUL-Parsers'!

!SoulParseTreeBuilder class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
forParser: parser
	
	^super new forParser: parser! !

!SoulParseTreeBuilder class methodsFor: 'instance creation' stamp: 'sp 24/10/11 15:26'!
new
	
	^self forParser: nil! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
addKeyword: keyword andArg: argument to: keywordMessageCompound
	
	^keywordMessageCompound addKeyword: keyword term: argument! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
addKeyword: keyword term: term toKeywordedCompound: compound
	
	^compound addKeyword: keyword term: term! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
addTabledRules: aSoulTabledProgramSequence inFront: aSoulProgramSequence
	
	^aSoulTabledProgramSequence addAll: aSoulProgramSequence! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
addTerm: aTerm inFront: aTermSequence
	
	^aTermSequence addTermFirst: aTerm! !

!SoulParseTreeBuilder methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
allAfter: anObject in: aCollection
	
	| startIndex |
	startIndex := aCollection indexOf: anObject.
	^aCollection
		copyFrom: startIndex + 1
		to: aCollection size! !

!SoulParseTreeBuilder methodsFor: 'private' stamp: 'sp 24/10/11 15:26'!
allBefore: anObject in: aCollection
	"| stopIndex |
	stopIndex := aCollection indexOf: anObject."

	
	^aCollection copyUpTo: anObject! !

!SoulParseTreeBuilder methodsFor: 'building annotated SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildAnnotatedTerm: aTerm withAnnotation: anAnnotationTerm
	
	^SoulFactory current makeAnnotatedTerm term: aTerm annotation:
			anAnnotationTerm! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildCallTerm: aTerm variable: aVariable
	"I made a mistake in the parser generation... the actual arguments that are passed are switched with respect to the keywords..."

	
	^SoulFactory current makeCallTerm variable: aVariable term: aTerm! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildCompound: aFunctor withTerms: aTermSequence
	
	^SoulFactory current makeCompound functor: aFunctor termSequence:
			aTermSequence! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildConstant: anIdentifierString
	
	^SoulFactory current makeConstant name: anIdentifierString asSymbol! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildCut
	
	^SoulFactory current makeCut new! !

!SoulParseTreeBuilder methodsFor: 'DCG' stamp: 'sp 24/10/11 15:26'!
buildDCGRule: head body: body
	
	^DCGRule condition: body conclusion: head! !

!SoulParseTreeBuilder methodsFor: 'building SmalltalkTerm/QuotedCode' stamp: 'sp 24/10/11 15:26'!
buildDecoratedVariable: aName with: aCollectionOfConstants
	
	^(DecoratedVariable name: (aName allButFirst: 1)) decorators:
			aCollectionOfConstants! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildDelayedVariable: anIdentifierString
	
	^SoulFactory current makeDelayedVariable
		name: (anIdentifierString allButFirst: 1)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildEmptyList
	
	^SoulFactory current makeEmptyList new! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildEmptyList: identifier
	
	^SoulFactory current makeEmptyList new! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildFact: aTerm
	
	^(SoulFactory current makeFact conclusion: aTerm)
		comment: self lastComments;
		yourself! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildKeyword: anIdentifierString
	
	^SoulFactory current makeConstant
		name: (anIdentifierString allButLast: 1) asSymbol! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildKeywordedCompound: keyword firstTerm: term
	
	^SoulFactory current makeMessageTerm
		functor: (SoulFactory current makeKeywordFunctor keyword: keyword)
		termSequence: (self buildTermSequence: term)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildKeywordedMessageFirstKey: keyword andArgument: argument
	
	^SoulFactory current makeMessageTerm
		functor: (SoulFactory current makeMessageFunctor keyword: keyword)
		termSequence: (self buildTermSequence: argument)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
buildKeywordedMessageReceiver: receiver message: keywordedMessageCompound
	
	^keywordedMessageCompound addTermFirst: receiver! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildList: aTermSequence
	"take a look at this... we should avoid unwrapping of the collection"

	
	^SoulFactory current makeList termSequence: aTermSequence! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildNegVariable: anIdentifierString
	
	^SoulFactory current makeNegVariable name: (anIdentifierString allButFirst: 2)! !

!SoulParseTreeBuilder methodsFor: 'DCG' stamp: 'sp 24/10/11 15:26'!
buildNormalbody: aBody on: aStream
	
	| counter |
	counter := 1.
	aBody terms
		do:
			[:aTerm | 
			aTerm printOn: aStream.
			aStream
				skip: -1;
				write: ',?tokens' , counter asString.
			counter := counter + 1.
			aStream write: ',?tokens' , counter asString , ')']
		separatedBy: [aStream write: ',']! !

!SoulParseTreeBuilder methodsFor: 'DCG' stamp: 'AndyKellens 11/2/2011 09:53'!
buildOldDCGRule: head body: body
	"Convert a DCG spec to a logic rule:
		s(a,b) --> np(a),vp(b)   ==>>  s(a,b,L1,L3) if np(a,L1,L2),vp(b,L2,L3)"

	
	| aStream |
	aStream := WriteStream on: String new.
	head printOn: aStream.
	head multiplicity > 0
		ifTrue:
			[aStream skip: -1.
			aStream write: ',']
		ifFalse: [aStream write: '('].
	aStream
		write: '?tokens1,?tokens' , (body terms size + 1) asString , ') if '.
	self convertDirectRuleCalls: body.
	body terms size = 1
		& (body terms first isKindOf: SoulFactory current makeList)
		ifTrue: [self buildTerminalbody: body on: aStream]
		ifFalse: [self buildNormalbody: body on: aStream].
	^(SoulFactory current makeParser
		parse: aStream contents
		onError:
			[:text :pos | self error: 'Parser error: ' , text , 'at: ' , pos asString])
		clauses first! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildPair: aTermSequence cdr: aTerm
	"take a look at this... we should avoid unwrapping of the collection"

	
	^SoulFactory current makePair new
		addChildrenInitial:
			((OrderedCollection new)
				add: aTermSequence;
				add: aTerm;
				yourself)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildPosVariable: anIdentifierString
	
	^SoulFactory current makePosVariable name: (anIdentifierString allButFirst: 2)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildProgramSequence: aHornClause
	
	^SoulFactory current makeProgramSequence new addTermFirst: aHornClause! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildQuery: anObject
	
	^SoulFactory current makeQuery condition: anObject! !

!SoulParseTreeBuilder methodsFor: 'building SmalltalkTerm/QuotedCode' stamp: 'AndyKellens 11/2/2011 09:53'!
buildQuotedCodeConstant: aString
	
	^SoulFactory current makeQuotedCodeConstant name: aString asSymbol! !

!SoulParseTreeBuilder methodsFor: 'building SmalltalkTerm/QuotedCode' stamp: 'AndyKellens 11/2/2011 09:53'!
buildQuotedCodeTerm: source
	
	^SoulFactory current makeQuotedCodeTerm
		source:
			(source
				copyFrom: 2
				to: source size - 1)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
buildRegexMessageReceiver: receiver message: keywordedMessageCompound
	
	| term |
	term := RegexTerm newFromMessageTerm: keywordedMessageCompound withGoals:
			receiver.
	^term generateStates! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildResultMessageTerm: message resultTerm: result
	
	^SoulFactory current makeResultMessageTerm message: message result: result! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildRuleConclusion: aTerm condition: aTermSequence
	
	^(SoulFactory current makeRule condition: aTermSequence conclusion: aTerm)
		comment: self lastComments;
		yourself! !

!SoulParseTreeBuilder methodsFor: 'building SmalltalkTerm/QuotedCode' stamp: 'AndyKellens 11/2/2011 09:53'!
buildSmalltalkTerm: source
	
	^SoulFactory current makeSmalltalkTerm
		source:
			(source
				copyFrom: 2
				to: source size - 1)! !

!SoulParseTreeBuilder methodsFor: 'building SmalltalkTerm/QuotedCode' stamp: 'AndyKellens 11/2/2011 09:53'!
buildSpliceCompound: aString
	
	| result |
	result := SoulFactory current makeParser
		parse: (aString allButFirst: 1)
		onError: [:ex :id | self error: ex , ' at ' , id asString].
	^result clauses first conclusion	"asApplyCompound"! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildSymbol: anIdentifierString
	
	^SoulFactory current makeConstant
		name:
			(anIdentifierString
				copyFrom: 2
				to: anIdentifierString size) asSymbol! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'sp 24/10/11 15:26'!
buildTabledProgramSequence: aSoulProgramSequence
	
	^aSoulProgramSequence! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'AndyKellens 10/28/2011 11:16'!
buildTabledRule: aSoulRule
	
	^aSoulRule
		asTabledRuleWithConditionString: aSoulRule condition asString! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'sp 24/10/11 15:26'!
buildTabledRuleBackup: aSoulRule
	
	| sequence |
	sequence := self buildProgramSequence: (self slgCallRuleFor: aSoulRule).
	sequence clauses addAll: (self slgImplementationRulesFor: aSoulRule).
	^sequence! !

!SoulParseTreeBuilder methodsFor: 'building SmalltalkTerm/QuotedCode' stamp: 'sp 24/10/11 15:26'!
buildTemplateQuery: source type: aType
	
	Transcript
		show:
				'WARNING: Template queries not supported yet in Squeak port of SOUL.';
		cr.
	^CompoundTerm
		functor: (Constant name: 'templateQueriesNotSupported')
		termSequence:
			(TermSequence terms: (Array with: (Constant name: 'dummy')))! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildTermSequence: aTerm
	
	^SoulFactory current makeTermSequence new addTermFirst: aTerm! !

!SoulParseTreeBuilder methodsFor: 'DCG' stamp: 'sp 24/10/11 15:26'!
buildTerminalbody: aBody on: aStream
	
	aStream write: 'equals(?tokens1,'.
	aBody terms first printOn: aStream.
	aStream skip: -1.
	aStream write: ' | ?tokens2>)'! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildUnaryMessageFromId: id
	
	^SoulFactory current makeUnaryMessageFunctor message: (self buildConstant: id)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildUnaryMessageFromVariable: var
	
	^SoulFactory current makeUnaryMessageFunctor message: var! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildUnaryMessageReceiver: receiver message: message
	
	^SoulFactory current makeMessageTerm
		functor: message
		termSequence: (SoulFactory current makeTermSequence term: receiver)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildUnderscoreVariable: anIdentifierString
	
	^SoulFactory current makeUnderscoreVariable new! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildVarArgsCompound: aFunctor withTerms: aTermSequence
	
	^SoulFactory current makeVarArgsCompound functor: aFunctor termSequence:
			aTermSequence! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildVarKeyword: anIdentifierString
	
	^SoulFactory current makeVariable
		name: ((anIdentifierString allButFirst: 1) allButLast: 1)! !

!SoulParseTreeBuilder methodsFor: 'building SOUL' stamp: 'AndyKellens 11/2/2011 09:53'!
buildVariable: anIdentifierString
	
	^SoulFactory current makeVariable name: (anIdentifierString allButFirst: 1)! !

!SoulParseTreeBuilder methodsFor: 'DCG' stamp: 'AndyKellens 11/2/2011 09:53'!
convertDirectRuleCalls: aBody
	"Process the direct calls to logic rules. They are embedded in quotedcode terms... a little bit of a mis-usage, but it works ;-)"

	
	| newTerms |
	newTerms := aBody terms
		collect:
			[:aClause | 
			(aClause isKindOf: QuotedCodeTerm)
				ifTrue:
					[SoulFactory current makeParser
						parse: aClause source
						onError: [:a :b | self error: 'Something went terribly wrong!!']]
				ifFalse: [aClause]].
	aBody terms: newTerms! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'sp 24/10/11 15:26'!
detectTabledCallIn: aTermCollection using: aConclusion
	
	| myLiteral |
	myLiteral := aConclusion predicate , aConclusion multiplicity asString.	"Check if the conditions contain a recursive call or a call to a tabled predicate. TODO: check for tabled predicate too"

	^aTermCollection
		detect:
			[:aCondition | 
			(aCondition functor printString , aCondition multiplicity printString) =
					myLiteral]
		ifNone: [nil]! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'sp 24/10/11 15:26'!
environmentVariablesForCallTerm: aCallOfTabledPredicate inRuleWithConclusion: aConclusion termsBefore: beforeCollection termsAfter: afterCollection
	"TODO: should be possible to capture the actual environment instead"

	
	| left right lfree rfree varsintersected |
	left := OrderedCollection new.
	left addAll: beforeCollection.
	left add: aConclusion.
	lfree := self variableNamesIn: (TermSequence terms: left).
	right := OrderedCollection new.
	right addAll: afterCollection.
	right add: aConclusion.
	right add: aCallOfTabledPredicate.
	rfree := self variableNamesIn: (TermSequence terms: right).
	varsintersected := (lfree select: [:v | rfree includes: v])
		asOrderedCollection.
	^ListTerm
		newFromCollection:
			(varsintersected collect: [:name | Variable name: name])! !

!SoulParseTreeBuilder methodsFor: 'initialization' stamp: 'sp 24/10/11 15:26'!
forParser: aParser
	
	parser := aParser! !

!SoulParseTreeBuilder methodsFor: 'comment handling' stamp: 'sp 24/10/11 15:26'!
lastComments
	
	^parser
		ifNil: ['']
		ifNotNil: [parser giveAndWipeComment]! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'AndyKellens 11/2/2011 09:53'!
slgAnswerRuleFor: originalSoulTerm sequenceNr: anInteger callTerm: resultTerm conditions: aTermCollection environmentVariables: vars
	
	| answer |
	answer := originalSoulTerm conclusion.
	^(SoulFactory current makeParser
		parse:
			originalSoulTerm predicate , 'cont' , anInteger asString , '(?Id,'
				, vars asString , ',' , resultTerm asString , ',D) if '
				,
					(aTermCollection isEmpty
						ifTrue: ['']
						ifFalse: [(TermSequence terms: aTermCollection) printString , ',']) ,
					'answer(?Id,' , answer asString , ',D)'
		onError: [:e :p | self halt]) clauses first! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'AndyKellens 11/2/2011 09:53'!
slgCallRuleFor: aSoulRule
	
	| literal |
	literal := aSoulRule conclusion printString.
	^(SoulFactory current makeParser
		parse: literal , ' if slg(' , literal , ')'
		onError: [:a :b | self error: 'Something went terribly wrong!!']) clauses
		first asTabledRuleWithConditionString: aSoulRule condition displayString! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'AndyKellens 11/2/2011 09:53'!
slgContinuationRulesFor: originalSoulRule sequenceNr: anInteger callTerm: resultTerm remainingConditions: aTermCollection endVars: endVars
	
	| tabledCall splitIndex ownConditions contConditions rules vars |
	tabledCall := self
		detectTabledCallIn: aTermCollection
		using: originalSoulRule conclusion.
	tabledCall isNil
		ifTrue:
			[^OrderedCollection
				with:
					(self slgAnswerRuleFor: originalSoulRule sequenceNr: anInteger callTerm:
							resultTerm conditions: aTermCollection environmentVariables: endVars)]
		ifFalse:
			[splitIndex := aTermCollection indexOf: tabledCall.
			ownConditions := splitIndex = 1
				ifTrue: [ownConditions := OrderedCollection new]
				ifFalse: [aTermCollection copyUpTo: splitIndex - 1].
			contConditions := aTermCollection
				copyFrom: splitIndex + 1
				to: aTermCollection size.
			vars := self
				environmentVariablesForCallTerm: tabledCall
				inRuleWithConclusion: originalSoulRule conclusion
				termsBefore: ownConditions
				termsAfter: contConditions.
			rules := self
				slgContinuationRulesFor: originalSoulRule
				sequenceNr: anInteger + 1
				callTerm: tabledCall
				remainingConditions: contConditions
				endVars: vars copy.	"rules
				addFirst: (Soul.Factory current makeParser
						parse: originalSoulRule predicate , 'cont' , anInteger asString
								, '(?Id,?Vars,' , resultTerm asString
								, ',D) if ' , (TermSequence terms: ownConditions) printString
						onError: [:e | self halt]) clauses
						first."

			rules
				addFirst:
					(SoulFactory current makeParser
						parse:
							originalSoulRule predicate , 'cont' , anInteger asString , '(?Id,'
								, endVars asString , ',' , resultTerm asString , ',D) if '
								, (TermSequence terms: ownConditions) printString , 'slgcall(?Id, '
								, vars asString , ',' , tabledCall printString , ',0,'
								, (originalSoulRule predicate , 'cont' , (anInteger + 1) asString)
								, ')'
						onError: [:e | self halt]) clauses first.
			^rules]! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'AndyKellens 11/2/2011 09:53'!
slgImplementationRulesFor: aSoulRule
	
	| tabledCall rules conclusionString ruleString previousConditionsString remainingConditions previousConditions vars |
	conclusionString := 'slg'
		, (aSoulRule conclusion printString allButLast: 1) , ',?Id)'.
	tabledCall := self
		detectTabledCallIn: aSoulRule condition terms
		using: aSoulRule conclusion.
	tabledCall isNil
		ifTrue:
			[ruleString := conclusionString , ' if ' , aSoulRule condition printString
				, ',answer(?Id,' , aSoulRule conclusion printString , ',0)'.
			rules := OrderedCollection new]
		ifFalse:
			["If a call to a tabled predicate (or a recursive call is present)"

			remainingConditions := self
				allAfter: tabledCall
				in: aSoulRule condition terms.
			previousConditions := self
				allBefore: tabledCall
				in: aSoulRule condition terms.
			vars := self
				environmentVariablesForCallTerm: tabledCall
				inRuleWithConclusion: aSoulRule conclusion
				termsBefore: previousConditions
				termsAfter: remainingConditions.
			rules := self slgContinuationRulesFor: aSoulRule sequenceNr: 0 callTerm:
					tabledCall remainingConditions: remainingConditions endVars: vars.
			previousConditionsString := (TermSequence terms: previousConditions)
				printString.
			previousConditionsString = ''
				ifFalse: [previousConditionsString := previousConditionsString , ','].
			ruleString := conclusionString , ' if ' , previousConditionsString ,
					'slgcall(?Id, ' , vars asString , ',' , tabledCall printString , ',0,'
				, aSoulRule predicate asString , 'cont0)'].
	rules
		addFirst:
			(SoulFactory current makeParser
				parse: ruleString
				onError: [:a :b | self error: 'Something went terribly wrong!!'])
				clauses first.
	^rules! !

!SoulParseTreeBuilder methodsFor: 'SLG' stamp: 'AndyKellens 11/2/2011 09:53'!
variableNamesIn: aTerm
	
	| dict variableVisitor |
	dict := Dictionary new.
	variableVisitor := SoulFactory current makeNamedVariableVisitor
		doBlock:
			[:var | 
			var isHiddenVariable
				ifFalse:
					[dict
						at: var name asSymbol
						ifAbsentPut: [var name asSymbol]]].
	aTerm accept: variableVisitor.
	^dict values asSet! !
QuotedCodeSmalltalkTokenScanner initialize!
